<?php
/**
 * This file contains a class 'WRLD_Common_Functions' which contains common menthods used.
 *
 * @package learndash-reports-by-wisdmlabs
 */

if ( ! class_exists( 'WRLD_Common_Functions' ) ) {
	/**
	 * Class that sets up all the LearnDash endpoints
	 *
	 * @author     WisdmLabs
	 * @since      1.0.0
	 * @subpackage LearnDash API
	 */
	class WRLD_Common_Functions {

		/**
		 * This function fetches the users of the learndash group specified by the parameter group_id,
		 * extracts their user Ids returns an array.
		 *
		 * @param int $group_id Id of the learndash group.
		 * @return array $group_user_ids array of user ids.
		 */
		public static function get_ld_group_user_ids( $group_id = 0 ) {
			if ( empty( $group_id ) || $group_id < 1 ) {
				return array();
			}

			$group_user_ids = array();
			$group_users    = learndash_get_groups_users( $group_id );

			if ( ! empty( $group_users ) ) {
				foreach ( $group_users as $user ) {
					$group_user_ids[] = $user->ID;
				}
			}
			return $group_user_ids;
		}


		/**
		 * The function gets a $ date in format "Y-m-d H:i:s".
		 * and tells wether the date is todays or not.
		 *
		 * @param string $date - in Y-m-d H:i:s format
		 * @return bool whether the its a todays date.
		 */
		public static function is_today( $date ) {
			if ( ! empty( $date ) ) {
				$today = new \DateTime(); // This object represents current date/time
				$today->setTime( 0, 0, 0 ); // reset time part, to prevent partial comparison
				$match_date = \DateTime::createFromFormat( 'Y-m-d H:i:s', $date );

				if ( empty( $match_date ) ) {
					return false;
				}

				$match_date->setTime( 0, 0, 0 ); // reset time part, to prevent partial comparison
				$diff      = $today->diff( $match_date );
				$diff_days = (int) $diff->format( '%R%a' );

				if ( 0 === $diff_days ) {
					return true;
				}
			}
			return false;
		}


		/**
		 * This function when passed with the request data ($_POST/$_GET)
		 * picks up the report related request data & returns an associative array
		 * of request parameters.
		 *
		 * @param array $request
		 * @return array $params an associative array of request parameteres for reporting plugin.
		 */
		public static function get_request_params( $request ) {
			$request_params = array();

			$reporting_filters = array(
				'start_date',
				'end_date',
				'category',
				'group',
				'course',
				'lesson',
				'topic',
				'learner',
			);

			foreach ( $reporting_filters as $filter ) {
				$request_params[ $filter ] = ! empty( $request[ $filter ] ) && 'null' != $request[ $filter ] ? sanitize_text_field( $request[ $filter ] ) : '';
			}

			return $request_params;
		}

		/**
		 * This function when passed with the start date & end date timestamps
		 * returns the duration i.e array of start date & end date along with the
		 * past duration start date & end dates.
		 *
		 * @param string $start_date
		 * @param string $end_date
		 * @return array $duration_data array('start_date','end_date','prev_start_date', 'prev_end_date')
		 */
		public static function get_duration_data( $start_date, $end_date, $format = 'Y-m-d H:i:s' ) {
			$duration_data = array();
			//if start date or end dates are not specified consider the duration of past 1 month.
			if ( empty( $start_date ) || empty( $end_date ) ) {
				$start_date = strtotime( '-1 month' );
				$end_date   = current_time( 'timestamp' );
			}

			$duration_data['start_date'] = date( $format , (int) $start_date );// phpcs:ignore.
			$duration_data['end_date']   = date( $format , (int) $end_date );// phpcs:ignore.
			$duration_data['end_date']   = self::is_today( $duration_data['end_date'] ) ? date( $format, current_time( 'timestamp' ) ) : $duration_data['end_date']; // phpcs:ignore.

			// Calculate previous similar period date range.
			$start_date_obj = new DateTime( $duration_data['start_date'] );
			$end_date_obj   = new DateTime( $duration_data['end_date'] );
			$date_range     = $end_date_obj->diff( $start_date_obj )->format( '%a' );
			$date_range     = ++$date_range;// because difference between 2021-11-29 18:30:00 and 2021-11-30 18:29:59 is 1 day not 0.

			$duration_data['prev_start_date'] = date( $format, strtotime( '-' . $date_range . ' days', strtotime( $duration_data['start_date'] ) ) );// phpcs:ignore.
			$duration_data['prev_end_date']   = date( $format, strtotime( '-' . $date_range . ' days', strtotime( $duration_data['end_date'] ) ) );// phpcs:ignore.

			return $duration_data;
		}


		/**
		 * This function when called gets the user roles for currently logged in user,
		 * categorize the roles in 'Administrator', 'Group Leader', 'Instructor' & 'Learner'
		 * & return the category in order to define data accesibility.
		 *
		 * @return string administrator | group Leader | instructor | learner
		 */
		public static function get_current_user_role_access() {
			$user  = wp_get_current_user(); // getting & setting the current user
			$roles = ! empty( $user ) ? (array) $user->roles : array();
			$role  = false;
			if ( ! empty( $roles ) ) {
				if ( in_array( 'administrator', $roles, true ) ) {
					$role = 'administrator';
				} elseif ( in_array( 'group_leader', $roles, true ) && in_array( 'wdm_instructor', $roles, true ) ) {
					$role = 'group_leader_instructor';
				} elseif ( in_array( 'group_leader', $roles, true ) ) {
					$role = 'group_leader';
				} elseif ( in_array( 'wdm_instructor', $roles, true ) ) {
					$role = 'instructor';
				} else {
					$role = 'learner';
				}
			}

			return apply_filters( 'wrld_filter_accessibility_user_role', $role );
		}

		/**
		 * Returns the Ids of the groups managed by the user specified by the
		 * user Id
		 *
		 * @param int $user_id
		 * @return array $group_ids
		 */
		public static function get_managed_group_ids( $user_id ) {
			$group_ids = array();
			if ( ! empty( $user_id ) ) {
				$group_ids = learndash_get_administrators_group_ids( $user_id );
			}
			return $group_ids;
		}


		/**
		 * Finds the users enrolled in each of the group Ids  & returns the list of unique
		 * user Ids from results.
		 *
		 * @param array $group_ids array of learndash course group Ids
		 * @return array $users array of users combinely present in group Ids mentioned.
		 */
		public static function get_users_enrolled_in_groups( $group_ids ) {
			$user_ids = array();
			if ( ! empty( $group_ids ) ) {
				foreach ( $group_ids as $group_id ) {
					$user_ids = array_unique( array_merge( $user_ids, self::get_ld_group_user_ids( $group_id ) ) );
				}
			}
			return $user_ids;
		}


		/**
		 * Finds the courses in each of the group Ids  & returns the list of unique
		 * course Ids from results.
		 *
		 * @param array $group_ids array of learndash course group Ids
		 * @return array $courses array of course Ids present in group Ids mentioned.
		 */
		public static function get_list_of_courses_in_groups( $group_ids ) {
			$course_ids = array();
			if ( ! empty( $group_ids ) ) {
				foreach ( $group_ids as $group_id ) {
					$course_ids = array_unique( array_merge( $course_ids, learndash_group_enrolled_courses( $group_id ) ) );
				}
			}
			return $course_ids;
		}

		/**
		 * Finds the users enrolled in each of the course Ids  & returns the list of unique
		 * user Ids from results.
		 *
		 * @param array $course_ids array of learndash course Ids
		 * @return array $enrolled_user_ids array of user Ids enrolled in all the courses.
		 */
		public static function get_list_of_users_enrolled_in_courses( $course_ids = array() ) {
			$enrolled_user_ids = array();
			if ( ! empty( $course_ids ) ) {
				foreach ( $course_ids as $course_id ) {
					$course_learners   = learndash_get_users_for_course( $course_id, array(), false );
					$course_learners   = is_array( $course_learners ) ? $course_learners : $course_learners->get_results();
					$enrolled_user_ids = array_unique( array_merge( $enrolled_user_ids, $course_learners ) );
				}
			}
			return $enrolled_user_ids;
		}


		/**
		 * Based on the user & user role for accessing reports provided this function returns the lst of course ids whose report can be
		 * accessible by the currently logged in user or the user specified with the $user_id
		 *
		 * @param int $user_id Id of the user for which we require to get the list of accessible users.
		 * @param string $user_role_for_access User role according to which we need to find the accesibility default:student.
		 * @param string $report_name name of the report for which we need accesssible users (Used while applying filter)
		 * @return array $accessible_courses arry of course ids for which the specified user can access the data.
		 */
		public static function get_accessible_courses_for_the_user( $user_id, $user_role_for_access = 'student', $report_name = null ) {
			$accessible_courses = array();

			if ( empty( $user_id ) ) {
				$user_id = get_current_user_id();
			}

			switch ( $user_role_for_access ) {
				case 'administrator':
					$accessible_courses = null;
					break;
				case 'group_leader_instructor':
					# The user is both a group leader & an Instructor hence Can access data of group users and courses users of the instructor.
					$groups_managed     = self::get_managed_group_ids( $user_id );
					$accessible_courses = self::get_list_of_courses_in_groups( $groups_managed );
					$accessible_courses = function_exists( 'ir_get_instructor_complete_course_list' ) ? array_unique( array_merge( $accessible_courses, ir_get_instructor_complete_course_list( $user_id ) ) ) : $accessible_courses;
					break;
				case 'group_leader':
					# Can access data of limited users and courses belongs to the groups managed by user
					$groups_managed     = self::get_managed_group_ids( $user_id );
					$accessible_courses = self::get_list_of_courses_in_groups( $groups_managed );
					break;
				case 'instructor':
					$accessible_courses = function_exists( 'ir_get_instructor_complete_course_list' ) ? ir_get_instructor_complete_course_list( $user_id ) : learndash_user_get_enrolled_courses( $user_id, array(), false );
					break;
				case 'student':
					$accessible_courses = learndash_user_get_enrolled_courses( $user_id, array(), false );
					break;
				default:
					# No access to the data
					break;
			}
			return apply_filters( 'wrld_course_accessibility_for_reports', $accessible_courses, $user_id, $user_role_for_access, $report_name );
		}


		/**
		 * Based on the user & user role for accessing reports provided this function returns the lst of users whose report can be
		 * accessible by the currently logged in user or the user specified with the $user_id
		 *
		 * @param int $user_id Id of the user for which we require to get the list of accessible users.
		 * @param string $user_role_for_access User role according to which we need to find the accesibility default:student.
		 * @param string $report_name name of the report for which we need accesssible users (Used while applying filter)
		 * @return array $accessible_users arry of user Ids for which the specified user can access the data.
		 */
		public static function get_accessible_users_for_the_user( $user_id, $user_role_for_access = 'student', $report_name = null ) {
			$accessible_users = array();
			switch ( $user_role_for_access ) {
				case 'administrator':
					$accessible_users = null;
					break;
				case 'group_leader_instructor':
					# The user is both a group leader & an Instructor hence Can access data of group users and courses users of the instructor.
					$groups_managed     = self::get_managed_group_ids( get_current_user_id() );
					$accessible_users   = self::get_users_enrolled_in_groups( $groups_managed );
					$instructor_courses = function_exists( 'ir_get_instructor_complete_course_list' ) ? ir_get_instructor_complete_course_list( get_current_user_id() ) : array();
					$accessible_users   = array_unique(
						array_merge(
							$accessible_users,
							self::get_list_of_users_enrolled_in_courses( $instructor_courses )
						)
					);
					break;
				case 'group_leader':
					# Can access data of limited users and courses belongs to the groups managed by user
					$groups_managed   = self::get_managed_group_ids( get_current_user_id() );
					$accessible_users = self::get_users_enrolled_in_groups( $groups_managed );
					break;
				case 'instructor':
					$accessible_courses = function_exists( 'ir_get_instructor_complete_course_list' ) ? ir_get_instructor_complete_course_list( get_current_user_id() ) : learndash_user_get_enrolled_courses( $user_id, array(), false );
					$accessible_users   = self::get_list_of_users_enrolled_in_courses( $accessible_courses );
					break;
				case 'student':
					$accessible_users = array( $user_id );
					break;
				default:
					# No access to the data
					break;
			}
			return apply_filters( 'wrld_user_accessibility_for_reports', $accessible_users, $user_id, $user_role_for_access, $report_name );
		}


		public static function get_values_for_request_params( $request_data = array() ) {
			$request_data_values = array();
			if ( ! empty( $request_data ) ) {
				foreach ( $request_data as $key => $value ) {
					if ( ( 'course' == $key || 'group' == $key || 'lesson' == $key || 'topic' == $key ) && '' != $value ) {
						$request_data_values[ $key ] = str_replace( '&#8211;', '-', get_the_title( $value ) );
					} elseif ( 'category' == $key && '' != $value ) {
						$category                    = get_term_by( 'id', $value, 'ld_course_category' );
						$request_data_values[ $key ] = ! empty( $category ) ? $category->name : '';
					} elseif ( 'learner' == $key ) {
						$user_data                   = get_userdata( $value );
						$request_data_values[ $key ] = ! empty( $user_data ) ? $user_data->display_name : '';
					} else {
						$request_data_values[ $key ] = $value;
					}
				}
			}

			return apply_filters( 'wrld_filter_values_for_request_params', $request_data_values, $request_data );
		}

	}
}
