<?php
/**
 * This file contains a class that is used to setup the LearnDash endpoints.
 *
 * @package learndash-reports-by-wisdmlabs
 */

require_once 'class-wrld-common-functions.php';
/**
 * Class that sets up all the LearnDash endpoints
 *
 * @author     WisdmLabs
 * @since      1.0.0
 */
class WRLD_Course_Progress_Info extends WRLD_Common_Functions {

	/**
	 * This static contains the number of points being assigned on course completion
	 *
	 * @var    Instance of WRLD_Revenue_API class
	 * @since  1.0.0
	 * @access private
	 */
	private static $instance = null;

	/**
	 * This static method is used to return a single instance of the class
	 *
	 * @since  1.0.0
	 * @access public
	 * @return Object
	 */
	public static function get_instance() {
		if ( empty( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * This is a constructor which will be used to initialize required hooks
	 *
	 * @since  1.0.0
	 * @access private
	 * @see    initHook static method
	 */
	private function __construct() {
	}

	/**
	 * Gets the time spent by user in the quiz.
	 *
	 * Total of each started/complete time set.
	 *
	 * @since 1.0.0
	 *
	 * @param int $user_id   Optional. The ID of the user to get quiz time spent. Default 0.
	 * @param int $quiz_id Optional. The ID of the quiz to get time spent. Default 0.
	 *
	 * @return int Total number of seconds spent.
	 */
	public function learndash_get_user_quiz_attempts_time_spent( $user_id, $quiz_id ) {
		$total_time_spent = 0;

		$attempts = learndash_get_user_quiz_attempts( $user_id, $quiz_id );
		if ( ( ! empty( $attempts ) ) && ( is_array( $attempts ) ) ) {
			foreach ( $attempts as $attempt ) {
				if ( empty( $attempt->activity_completed ) || empty( $attempt->activity_started ) ) {
					continue;
				}
				if ( $attempt->activity_completed - $attempt->activity_started < 0 ) {
					continue;
				}
				$total_time_spent += ( $attempt->activity_completed - $attempt->activity_started );
			}
		}

		return $total_time_spent;
	}

	/**
	 * Gets the time spent by user in the course.
	 *
	 * Total of each started/complete time set.
	 *
	 * @since 1.0.0
	 *
	 * @param int $user_id   Optional. The ID of the user to get course time spent. Default 0.
	 * @param int $course_id Optional. The ID of the course to get time spent. Default 0.
	 *
	 * @return int Total number of seconds spent.
	 */
	public function learndash_get_user_course_attempts_time_spent( $user_id = 0, $course_id = 0 ) {
		$total_time_spent = 0;

		$attempts = learndash_get_user_course_attempts( $user_id, $course_id );

		// We should only ever have one entry for a user+course_id. But still we are returned an array of objects.
		if ( ( ! empty( $attempts ) ) && ( is_array( $attempts ) ) ) {
			foreach ( $attempts as $attempt ) {

				if ( ! empty( $attempt->activity_completed ) ) {
					// If the Course is complete then we take the time as the completed - started times.
					if ( empty( $attempt->activity_completed ) || empty( $attempt->activity_started ) ) {
						continue;
					}
					if ( $attempt->activity_completed - $attempt->activity_started < 0 ) {
						continue;
					}
					$total_time_spent += ( $attempt->activity_completed - $attempt->activity_started );
				} else {
					if ( empty( $attempt->activity_updated ) || empty( $attempt->activity_started ) ) {
						continue;
					}
					if ( $attempt->activity_updated - $attempt->activity_started < 0 ) {
						continue;
					}
					// But if the Course is not complete we calculate the time based on the updated timestamp.
					// This is updated on the course for each lesson, topic, quiz.
					$total_time_spent += ( $attempt->activity_updated - $attempt->activity_started );
				}
			}
		}

		return $total_time_spent;
	}

	/**
	 * This method is used as a API callback to return day-to-day enrollments data.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_daily_enrollments() {
		global $wpdb;
		$request_data = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data = self::get_request_params( $request_data );
		$duration     = self::get_duration_data( $request_data['start_date'], $request_data['end_date'], 'Y-m-d' );

		// Loop through all the days in the required time-period.
		$begin = new DateTime( $duration['start_date'] );
		$begin = $begin->modify( '+1 day' );
		$end   = new DateTime( $duration['end_date'] );
		$end   = $end->modify( '+1 day' );

		$interval = new DateInterval( 'P1D' );
		$period   = new DatePeriod( $begin, $interval, $end );

		$enrollments      = array();
		$total_enrollment = 0;
		$days             = 0;
		foreach ( $period as $date ) {
			$days++;
			// Get the timestamp of the target date's start and end.
			$target_date = $date->format( 'Y-m-d' );
			$day_start   = $date->format( 'Y-m-d 00:00:00' );
			$day_end     = $date->format( 'Y-m-d 23:59:59' );

			// Fetch enrollment activity between day start and day end.
			//$course_enrolls = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT( * ) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'user_activity' ) ) . ' WHERE activity_type=%s AND activity_started BETWEEN %d AND %d LIMIT 1', 'access', $day_start, $day_end ) );
			$course_enrolls = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(*) as count FROM ' . $wpdb->users . ' as users  JOIN ' . $wpdb->usermeta . " as usermeta ON users.id=usermeta.user_id WHERE users.user_registered>=%s AND users.user_registered<=%s AND usermeta.meta_key='wp_capabilities' AND usermeta.meta_value NOT LIKE %s", $day_start, $day_end, '%administrator%' ) );

			$total_enrollment = $total_enrollment + $course_enrolls;
			$enrollments[]    = array(
				'count' => $course_enrolls,
				'date'  => $target_date,
			);
		}
		// Calculate average Enrollment.
		$average_enrollment = round( $total_enrollment / $days );
		return new WP_REST_Response(
			array(
				'requestData'       => self::get_values_for_request_params( $request_data ),
				'enrollments'       => $enrollments,
				/* translators: %1d:Average Enrollments*/
				'averageEnrollment' => 1 === $average_enrollment ? sprintf( __( '%d Learner', 'learndash-reports-by-wisdmlabs' ), $average_enrollment ) : sprintf( __( '%d Learners', 'learndash-reports-by-wisdmlabs' ), $average_enrollment ),
			),
			200
		);
	}

	/**
	 * This method returns the pending assignments count.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_pending_assignments_info() {
		$pending_count = learndash_get_assignments_pending_count();
		if ( empty( $pending_count ) || ! defined( 'LDRP_PLUGIN_VERSION' ) ) {
			$pending_count = 0;
		}
		return new WP_REST_Response(
			array(
				'pendingAssignments' => $pending_count,
			),
			200
		);
	}

	/**
	 * This method returns the Quiz completion rate based on the input parameters.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_quiz_completion_rate() {
		// Get Inputs.
		$request_data = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data = self::get_request_params( $request_data );
		unset( $request_data['start_date'] );
		unset( $request_data['end_date'] );
		$user_role_access   = self::get_current_user_role_access();
		$accessible_courses = self::get_accessible_courses_for_the_user( get_current_user_id(), $user_role_access, 'quiz_completion_rate' );
		$accessible_users   = self::get_accessible_users_for_the_user( get_current_user_id(), $user_role_access, 'quiz_completion_rate' );
		$category_query     = array();
		// Learner Filters.
		if ( isset( $request_data['learner'] ) && ! empty( $request_data['learner'] ) ) {
			$courses = learndash_user_get_enrolled_courses( $request_data['learner'], array(), false );
			if ( isset( $accessible_courses ) ) {
				$courses = array_intersect( $courses, $accessible_courses );
			}
			if ( empty( $courses ) ) {
				return new WP_Error(
					'no-data',
					sprintf(
						/* translators: %s: custom label for courses */
						__( 'No accessible %s found', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'courses' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$quiz_count       = 0;
			$total_percentage = 0;
			$course_data      = array();
			$completed_count  = 0;
			foreach ( $courses as $course_id ) {
				$quizzes    = learndash_course_get_children_of_step( $course_id, $course_id, 'sfwd-quiz', 'ids', true );
				$quiz_count = $quiz_count + count( $quizzes );
				if ( empty( $quizzes ) ) {
					continue;
				}
				foreach ( $quizzes as $quiz ) {
					$percentage = 0;
					if ( learndash_is_quiz_complete( $request_data['learner'], $quiz, $course_id ) ) {
						$percentage = 100;
						$completed_count++;
					}
					$total_percentage = $total_percentage + $percentage;

					$course_data[ $quiz ] = array(
						'average_completion' => $percentage,
						'title'              => get_the_title( $quiz ),
					);
				}
			}
			if ( $quiz_count > 0 ) {
				$average_completion = floatval( number_format( $total_percentage / $quiz_count, 2, '.', '' ) );// Cast to integer if no decimals.
			} else {
				$average_completion = 0;
			}
			return new WP_REST_Response(
				array(
					'requestData'               => self::get_values_for_request_params( $request_data ),
					'average_completion'        => $average_completion,
					'total_quiz_completion'     => $completed_count,
					'total_quizzes'             => $quiz_count,
					'coursewise_statistics_new' => $course_data,
				),
				200
			);
		} elseif ( isset( $request_data['course'] ) && ! empty( $request_data['course'] ) ) {
			if ( isset( $accessible_courses ) ) {
				if ( ! in_array( $request_data['course'], $accessible_courses ) ) {
					return new WP_Error(
						'unauthorized',
						sprintf(/* translators: %s: custom label for course */
							__( 'You don\'t have access to this %s.', 'learndash-reports-by-wisdmlabs' ),
							\LearnDash_Custom_Label::label_to_lower( 'course' )
						),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					);
				}
			}
			$course        = get_post( $request_data['course'] );
			$lesson        = $request_data['lesson'];
			$topic         = $request_data['topic'];
			$child_step_id = $course->ID;
			$child_step_id = ! empty( $lesson ) ? (int) $lesson : $child_step_id;
			$child_step_id = ! empty( $topic ) ? (int) $topic : $child_step_id;
			// Check for valid course.
			if ( empty( $course ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			// Get all students for a course.
			$students = learndash_get_users_for_course( $course->ID, array(), false ); // Third argument is $exclude_admin.
			$students = is_array( $students ) ? $students : $students->get_results();
			$students = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			// Check if any students enrolled.
			if ( empty( $students ) ) {
				return new WP_Error(
					'no-data',
					__( 'No Students enrolled', 'learndash-reports-by-wisdmlabs' ),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}

			$quizzes       = learndash_course_get_children_of_step( $course->ID, $child_step_id, 'sfwd-quiz', 'ids', true );
			$quiz_count    = count( $quizzes );
			$student_count = count( $students );
			if ( empty( $quizzes ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for quizzes */
						__( 'No %s found for the selected filters', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'quizzes' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);

			}
			$completed_count = 0;
			foreach ( $students as $student ) {
				$completed = 0;
				foreach ( $quizzes as $quiz ) {
					if ( learndash_is_quiz_complete( $student, $quiz, $course->ID ) ) {
						$completed++;
					}
				}
				$quiz_completion_rate_per_student[] = $completed / $quiz_count;
				if ( $quiz_count === $completed ) {
					$completed_count++;
				}
			}
			$average_completion = floatval( number_format( ( $completed_count / $student_count ) * 100, 2, '.', '' ) );// Cast to integer if no decimals.
			$completed_count    = $completed_count;
			$incomplete_count   = $student_count - $completed_count;
			return new WP_REST_Response(
				array(
					'requestData'              => self::get_values_for_request_params( $request_data ),
					'completed_count'          => $completed_count,
					'incomplete_count'         => $incomplete_count,
					'average_completion'       => $average_completion,
					'learner_count'            => $student_count,
					'learner_completion_count' => $completed_count,
					'quiz_count'               => $quiz_count,
				),
				200
			);
		} elseif ( isset( $request_data['category'] ) && ! empty( $request_data['category'] ) ) {
			// Check if course category enabled.
			if ( ! taxonomy_exists( 'ld_course_category' ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category disabled. Please contact admin.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			// Check if valid category passed.
			if ( ! is_object( get_term_by( 'id', $request_data['category'], 'ld_course_category' ) ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$category_query = array(
				array(
					'taxonomy'         => 'ld_course_category',
					'field'            => 'term_id',
					'terms'            => $request_data['category'], // Where term_id of Term 1 is "1".
					'include_children' => false,
				),
			);
		} elseif ( isset( $request_data['group'] ) && ! empty( $request_data['group'] ) ) {
			$group_courses      = learndash_group_enrolled_courses( $request_data['group'] );
			$accessible_courses = null != $accessible_courses ? array_intersect( $group_courses, $accessible_courses ) : $group_courses;
			$group_users        = self::get_ld_group_user_ids( $request_data['group'] );
			$accessible_users   = null != $accessible_users ? array_intersect( $group_users, $accessible_users ) : $group_users;
		}

		$args = array(
			'post_type'      => 'sfwd-courses',
			'posts_per_page' => '-1',
			'post__in'       => $accessible_courses,
		);

		if ( ! empty( $category_query ) ) {
			$args['tax_query'] = $category_query;
		}
		$courses     = get_posts( $args );
		$course_data = array();
		// Check if any courses present in the category.
		if ( empty( $courses ) || ( null != $accessible_courses && empty( $accessible_courses ) ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s data found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'quiz' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}
		// Average Completion percentage count variables.
		$course_count                    = count( $courses );
		$universal_quiz_completion_count = 0;
		$universal_quiz_count            = 0;
		$course_data                     = array();
		foreach ( $courses as $course ) {
			$students = learndash_get_users_for_course( $course->ID, array(), false ); // Third argument is $exclude_admin.
			$students = is_array( $students ) ? $students : $students->get_results();
			$students = null !== $accessible_users ? array_intersect( $accessible_users, $students ) : $students;

			if ( empty( $students ) ) {
				continue;
			}

			$quizzes               = learndash_course_get_children_of_step( $course->ID, $course->ID, 'sfwd-quiz', 'ids', true );
			$quiz_count            = count( $quizzes );
			$student_count         = count( $students );
			$universal_quiz_count += $quiz_count;
			// If no students in the course then the course has 0 percent completion.
			if ( empty( $quizzes ) ) {
				continue;
			}
			$completed_count = 0;
			foreach ( $students as $student ) {
				$completed = 0;
				foreach ( $quizzes as $quiz ) {
					if ( learndash_is_quiz_complete( $student, $quiz, $course->ID ) ) {
						$completed++;
						$universal_quiz_completion_count++;
					}
				}
				$quiz_completion_rate_per_student[] = $completed / $quiz_count;
				if ( $quiz_count === $completed ) {
					$completed_count++;
				}
			}
			$average_completion         = floatval( number_format( array_sum( $quiz_completion_rate_per_student ) / $student_count, 2, '.', '' ) );// Cast to integer if no decimals.
			$completed_count            = $completed_count;
			$incomplete_count           = $student_count - $completed_count;
			$course_data[ $course->ID ] = array(
				'completed_count'          => $completed_count,
				'incomplete_count'         => $incomplete_count,
				'average_completion'       => $average_completion,
				'learner_count'            => $student_count,
				'learner_completion_count' => $completed_count,
				'title'                    => get_the_title( $course->ID ),
			);
		}
		$average_completion       = 0;
		$learner_completion_count = 0;
		if ( empty( $course_data ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s data found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'quiz' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}
		foreach ( $course_data as $course_id => $course_info ) {
			$average_completion       = $average_completion + $course_info['average_completion'];
			$learner_completion_count = $learner_completion_count + $course_info['learner_completion_count'];
		}
		$average_completion = floatval( number_format( $average_completion / count( $course_data ), 2, '.', '' ) );// Cast to integer if no decimals.
		return new WP_REST_Response(
			array(
				'requestData'              => self::get_values_for_request_params( $request_data ),
				'average_completion'       => $average_completion,
				'learner_completion_count' => $learner_completion_count,
				'total_quiz_completion'    => $universal_quiz_completion_count,
				'total_quizzes'            => $universal_quiz_count,
				'coursewise_statistics'    => $course_data,
			),
			200
		);
	}

	/**
	 * This method returns quiz passing rate for quizzes of a course/lesson/topic.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_quiz_passing_rate() {
		// Get Inputs.
		$request_data       = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data       = self::get_request_params( $request_data );
		$duration           = self::get_duration_data( $request_data['start_date'], $request_data['end_date'], 'Y-m-d H:i:s' );
		$user_role_access   = self::get_current_user_role_access();
		$accessible_courses = self::get_accessible_courses_for_the_user( get_current_user_id(), $user_role_access, 'quiz_passing_rate' );
		$accessible_users   = self::get_accessible_users_for_the_user( get_current_user_id(), $user_role_access, 'quiz_passing_rate' );
		$category_query     = array();

		$attempts   = array();
		$start_date = $duration['start_date'];
		$end_date   = $duration['end_date'];
		if ( empty( $start_date ) || empty( $end_date ) ) {
			$start_date = strtotime( '-1 month' );
			$end_date   = current_time( 'timestamp' );
		}

		if ( null != $accessible_users && 0 === count( $accessible_users ) ) {
			return new WP_Error(
				'no-data',
				__( 'No learners found', 'learndash-reports-by-wisdmlabs' ),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		if ( null != $accessible_courses && 0 === count( $accessible_courses ) ) {
			return new WP_Error(
				'no-data',
				sprintf(
						/* translators: %s: custom label for course */
					__( 'No accessible %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::label_to_lower( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		if ( ! empty( $request_data['course'] ) ) {
			$attempts = $this->get_modulewise_activity( $request_data['course'], $request_data['course'], $start_date, $end_date, $accessible_users );
		}

		if ( ! empty( $request_data['lesson'] ) ) {
			$attempts = $this->get_modulewise_activity( $request_data['course'], $request_data['lesson'], $start_date, $end_date, $accessible_users );
		}

		if ( ! empty( $request_data['topic'] ) ) {
			$attempts = $this->get_modulewise_activity( $request_data['course'], $request_data['topic'], $start_date, $end_date, $accessible_users );
		}

		if ( ! empty( $request_data['topic'] ) || ! empty( $request_data['lesson'] ) || ! empty( $request_data['course'] ) ) {
			if ( isset( $accessible_courses ) ) {
				if ( ! in_array( $request_data['course'], $accessible_courses ) ) {
					return new WP_Error(
						'unauthorized',
						sprintf(/* translators: %s: custom label for course */
							__( 'You don\'t have access to this %s.', 'learndash-reports-by-wisdmlabs' ),
							\LearnDash_Custom_Label::label_to_lower( 'course' )
						),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					);
				}
			}
			if ( empty( $attempts ) ) {
				$total   = 0;
				$pass    = 0;
				$quizzes = 0;
				return new WP_REST_Response(
					array(
						'requestData'         => self::get_values_for_request_params( $request_data ),
						'quizwise_data'       => $attempts,
						'total_quiz_attempts' => $total,
						'pass_quiz_attempts'  => $pass,
						'avg_quiz_attempts'   => $total,
					),
					200
				);
			}
			$total   = 0;
			$pass    = 0;
			$quizzes = count( $attempts );
			$total   = array_sum( array_column( $attempts, 'total' ) );
			$pass    = array_sum( array_column( $attempts, 'pass' ) );
			$avg     = round( $total / $quizzes );

			return new WP_REST_Response(
				array(
					'requestData'         => self::get_values_for_request_params( $request_data ),
					'quizwise_data'       => $attempts,
					'total_quiz_attempts' => $total,
					'pass_quiz_attempts'  => $pass,
					'avg_quiz_attempts'   => $avg,
				),
				200
			);
		}

		$args = array(
			'post_type'      => 'sfwd-courses',
			'posts_per_page' => '-1',
		);
		// Now consider the scenario where a category is selected. (Pro feature).
		if ( isset( $request_data['category'] ) && ! empty( $request_data['category'] ) ) {
			// Check if course category enabled.
			if ( ! taxonomy_exists( 'ld_course_category' ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category disabled. Please contact admin.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			// Check if valid category passed.
			if ( ! is_object( get_term_by( 'id', $request_data['category'], 'ld_course_category' ) ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$args['tax_query'] = array(
				array(
					'taxonomy'         => 'ld_course_category',
					'field'            => 'term_id',
					'terms'            => $request_data['category'], // Where term_id of Term 1 is "1".
					'include_children' => false,
				),
			);
		}
		if ( isset( $request_data['group'] ) && ! empty( $request_data['group'] ) ) {
			$args['meta_query'] = array(
				array(
					'key'     => 'learndash_group_enrolled_' . $request_data['group'],
					'compare' => 'EXISTS',
				),
			);
		}
		$courses      = get_posts( $args );
		$course_data  = array();
		$course_count = count( $courses );
		// Check if any courses present in the category.
		if ( empty( $courses ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		$course_stats = array(
			'quiz_count'              => 0,
			'quiz_attempt_count'      => 0,
			'quiz_pass_attempt_count' => 0,
			'users_count'             => 0,
		);

		foreach ( $courses as $course ) {
			if ( isset( $accessible_courses ) ) {
				if ( ! in_array( $course->ID, $accessible_courses ) ) {
					--$course_count;
					continue;
				}
			}
			if ( count( learndash_course_get_children_of_step( $course->ID, $course->ID, 'sfwd-quiz', 'ids', true ) ) <= 0 ) {
				--$course_count;
				continue;
			}
			$attempts = $this->get_modulewise_activity( $course->ID, $course->ID, $start_date, $end_date, $accessible_users );
			if ( empty( $attempts ) ) {
				// $course_stats['quiz_attempt_count']       = $course_stats['quiz_attempt_count'] + 0;
				// $course_stats['users_count']              = $course_stats['users_count'] + 0;
				// $course_data[ $course->ID ]['percentage'] = 100;
				--$course_count;
				continue;
			}
			$course_data[ $course->ID ]['title']      = $course->post_title;
			$course_stats['quiz_count']               = $course_stats['quiz_count'] + count( learndash_course_get_children_of_step( $course->ID, $course->ID, 'sfwd-quiz', 'ids', true ) );
			$course_stats['quiz_attempt_count']       = $course_stats['quiz_attempt_count'] + array_sum( array_column( $attempts, 'total' ) );
			$course_stats['quiz_pass_attempt_count']  = $course_stats['quiz_pass_attempt_count'] + array_sum( array_column( $attempts, 'pass' ) );
			$course_stats['users_count']              = $course_stats['users_count'] + count( array_unique( array_merge( ...array_column( $attempts, 'users' ) ) ) );
			$course_data[ $course->ID ]['percentage'] = floatval( number_format( array_sum( array_column( $attempts, 'percentage' ) ) / count( $attempts ), 2, '.', '' ) );// Cast to integer if no decimals.
		}

		if ( $course_count <= 0 ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for quiz */
					__( 'No %s attempts found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'quiz' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		} else {
			$average_pass_rate = floatval( number_format( array_sum( array_column( $course_data, 'percentage' ) ) / $course_count, 2, '.', '' ) );// Cast to integer if no decimals.
		}

		return new WP_REST_Response(
			array(
				'requestData'             => self::get_values_for_request_params( $request_data ),
				'average_pass_rate'       => $average_pass_rate,
				'learner_count'           => $course_stats['users_count'],
				'quiz_attempt_count'      => $course_stats['quiz_attempt_count'],
				'quiz_pass_attempt_count' => $course_stats['quiz_pass_attempt_count'],
				'quiz_count'              => $course_stats['quiz_count'],
				'coursewise_data'         => $course_data,
			),
			200
		);
	}

	/**
	 * This method returns average quiz attempts by students for quizzes of a course/lesson/topic.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_average_quiz_attempts() {
		global $wpdb;
		// Get Inputs.
		$request_data       = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data       = self::get_request_params( $request_data );
		$user_role_access   = self::get_current_user_role_access();
		$accessible_courses = self::get_accessible_courses_for_the_user( get_current_user_id(), $user_role_access, 'average_quiz_attempts' );
		$accessible_users   = self::get_accessible_users_for_the_user( get_current_user_id(), $user_role_access, 'average_quiz_attempts' );
		$category_query     = array();
		$group_users        = null;//initialize with user ids if the user is group leader & have access to limited users

		$start_date = $request_data['start_date'];
		$end_date   = $request_data['end_date'];
		if ( empty( $start_date ) || empty( $end_date ) ) {
			$start_date = strtotime( '-1 month' );
			$end_date   = current_time( 'timestamp' );
		}

		if ( isset( $request_data['learner'] ) && ! empty( $request_data['learner'] ) ) {
			$courses = learndash_user_get_enrolled_courses( $request_data['learner'], array(), false );
			if ( isset( $accessible_courses ) ) {
				$courses = array_intersect( $courses, $accessible_courses );
			}
			if ( empty( $courses ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for courses */
						__( 'No accessible %s found', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'courses' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$quiz_attempt_data = array();
			$total_attempts    = 0;
			$quiz_count        = 0;
			foreach ( $courses as $course_id ) {
				$course_time = 0;
				$quizzes     = learndash_course_get_children_of_step( $course_id, $course_id, 'sfwd-quiz', 'ids', true );
				$quiz_count  = $quiz_count + count( $quizzes );
				if ( empty( $quizzes ) ) {
					continue;
				}
				foreach ( $quizzes as $quiz ) {
					// $count                      = learndash_get_user_quiz_attempts_count( $request_data['learner'], $quiz );
					$count          = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( \LDLMS_DB::get_table_name( 'user_activity' ) ) . ' WHERE user_id = %d AND post_id = %d AND activity_type = %s AND ( activity_completed >= %d AND activity_completed <= %d ) AND course_id=%d ORDER BY activity_id, activity_started ASC', $request_data['learner'], $quiz, 'quiz', $start_date, $end_date, $course_id ) );
					$total_attempts = $total_attempts + $count;
					if ( ! array_key_exists( $quiz, $quiz_attempt_data ) ) {
						$quiz_attempt_data[ $quiz ] = array(
							'count' => null == $count ? 0 : $count,
							'title' => get_the_title( $quiz ),
						);
					} else {
						$quiz_attempt_data[ $quiz ]['count'] += null == $count ? 0 : $count;
					}
				}
				unset( $quizzes );
			}
			if ( $quiz_count > 0 ) {
				$average_attempts = floatval( number_format( $total_attempts / $quiz_count, 2, '.', '' ) );// Cast to integer if no decimals.
			} else {
				$average_attempts = 0;
			}
			return new WP_REST_Response(
				array(
					'requestData'      => self::get_values_for_request_params( $request_data ),
					'quizwise_data'    => $quiz_attempt_data,
					'total_attempts'   => $total_attempts,
					'average_attempts' => $average_attempts,
					'total_quizzes'    => $quiz_count,
				),
				200
			);
		}

		if ( empty( $request_data['course'] ) ) {
			return new WP_Error(
				'invalid-input',
				sprintf(/* translators: %s: custom label for course */
					__( '%s selection is required', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'course' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		if ( isset( $accessible_courses ) ) {
			if ( ! in_array( $request_data['course'], $accessible_courses ) ) {
				return new WP_Error(
					'unauthorized',
					sprintf(/* translators: %s: custom label for course */
						__( 'You don\'t have access to this %s.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::label_to_lower( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
		}

		if ( ! empty( $request_data['topic'] ) ) {
			$quizzes = learndash_course_get_children_of_step( $request_data['course'], $request_data['topic'], 'sfwd-quiz', 'ids', true );
		} elseif ( ! empty( $request_data['lesson'] ) ) {
			$quizzes = learndash_course_get_children_of_step( $request_data['course'], $request_data['lesson'], 'sfwd-quiz', 'ids', true );
		} else {
			$quizzes = learndash_course_get_children_of_step( $request_data['course'], $request_data['course'], 'sfwd-quiz', 'ids', true );
		}
		if ( empty( $quizzes ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for quizzes */
					__( 'No %s found for the selected filters', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'quizzes' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}
		$quiz_attempt_data = array();
		$total_attempts    = 0;
		$quizzes_count     = count( $quizzes );
		$students          = learndash_get_users_for_course( $request_data['course'], array(), false );
		$students          = is_array( $students ) ? $students : $students->get_results();
		$students          = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
		$student_count     = count( $students );
		if ( empty( $students ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for course */
					__( 'No Students enrolled in the selected %s/module.', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::label_to_lower( 'course' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}
		$students = '"' . implode( '","', $students ) . '"';
		foreach ( $quizzes as $quiz ) {
			$quiz_pro_id = get_post_meta( $quiz, 'quiz_pro_id', true );
			$args        = array(
				'limit'   => 9999999999,
				'offset'  => 0,
				'order'   => 'DESC',
				'orderby' => 'create_time',
			);

			$placeholder_args = array();
			if ( ( false !== $start_date ) && ( false !== $end_date ) ) {
				$where    = 'statref.quiz_id="' . $quiz_pro_id . '"';
				$where   .= 'AND statref.create_time>="' . $start_date . '"';
				$where   .= 'AND statref.create_time<="' . $end_date . '"';
				$where   .= 'AND statref.user_id IN (' . $students . ')';
				$orderby  = ' ORDER BY ' . $args['orderby'] . ' ' . $args['order'];
				$limit    = ' LIMIT ' . $args['offset'] . ', ' . (int) $args['limit'];
				$attempts = $wpdb->get_var( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE ' . $where . $orderby . $limit ); // phpcs:ignore.
			} elseif ( ( false !== $start_date ) && ( false === $end_date ) ) {
				$where    = 'statref.quiz_id="' . $quiz_pro_id . '"';
				$where   .= 'AND statref.create_time>="' . $start_date . '"';
				$where   .= 'AND statref.user_id IN (' . $students . ')';
				$orderby  = ' ORDER BY ' . $args['orderby'] . ' ' . $args['order'];
				$limit    = ' LIMIT ' . $args['offset'] . ', ' . (int) $args['limit'];
				$attempts = $wpdb->get_var( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE ' . $where . $orderby . $limit ); // phpcs:ignore.
			} elseif ( ( false === $start_date ) && ( false !== $end_date ) ) {
				$where    = 'statref.quiz_id="' . $quiz_pro_id . '"';
				$where   .= 'AND statref.create_time<="' . $end_date . '"';
				$where   .= 'AND statref.user_id IN (' . $students . ')';
				$orderby  = ' ORDER BY ' . $args['orderby'] . ' ' . $args['order'];
				$limit    = ' LIMIT ' . $args['offset'] . ', ' . (int) $args['limit'];
				$attempts = $wpdb->get_var( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE ' . $where . $orderby . $limit ); // phpcs:ignore.

			} else {
				$where    = 'statref.quiz_id="' . $quiz_pro_id . '"';
				$where   .= 'AND statref.user_id IN (' . $students . ')';
				$orderby  = ' ORDER BY ' . $args['orderby'] . ' ' . $args['order'];
				$limit    = ' LIMIT ' . $args['offset'] . ', ' . (int) $args['limit'];
				$attempts = $wpdb->get_var( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE ' . $where . $orderby . $limit ); // phpcs:ignore.
			}

			$quiz_attempt_data[ $quiz ] = array(
				'title' => get_the_title( $quiz ),
				'count' => $attempts,
			);
			$total_attempts             = $total_attempts + $attempts;
		}
		$average_attempts = floatval( number_format( $total_attempts / $quizzes_count, 2, '.', '' ) );// Cast to integer if no decimals.
		return new WP_REST_Response(
			array(
				'requestData'      => self::get_values_for_request_params( $request_data ),
				'quizwise_data'    => $quiz_attempt_data,
				'total_attempts'   => $total_attempts,
				'average_attempts' => $average_attempts,
				'student_count'    => $student_count,
			),
			200
		);
	}

	/**
	 * This method is used to return Student-wise quiz activity for a module.
	 *
	 * @param  int $course     Course ID.
	 * @param  int $module     Lesson/Topic ID.
	 * @param  int $start_date Timestamp.
	 * @param  int $end_date   Timestamp.
	 * @return Array of Quiz related activity data.
	 */
	public function get_modulewise_activity( $course, $module, $start_date, $end_date, $accessible_users = null ) {
		$quizzes = learndash_course_get_children_of_step( $course, $module, 'sfwd-quiz', 'ids', true );
		if ( empty( $quizzes ) ) {
			return array();
		}
		$quizzes_str         = implode( ', ', $quizzes );
		$activity_query_args = array(
			'post_ids'        => $quizzes_str,
			'activity_type'   => 'quiz',
			'time_start'      => $start_date,// phpcs:ignore.
			'time_end'        => $end_date,// phpcs:ignore.
			'per_page'        => 1844674407370,
			'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
		);
		if ( null != $accessible_users ) {
			$activity_query_args['user_ids'] = $accessible_users;
		}
		$quiz_activity = learndash_reports_get_activity( $activity_query_args );
		$quiz_data     = array();
		$user          = wp_get_current_user();
		if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
			foreach ( $quiz_activity['results'] as $result ) {
				if ( ! current_user_can( 'manage_options' ) ) {
					if ( in_array( 'group_leader', (array) $user->roles, true ) ) {
						$group_users = learndash_get_course_groups_users_access( $course );
						if ( ! in_array( $result->user_id, $group_users ) ) {
							continue;
						}
					}
				}
				if ( ! isset( $quiz_data[ $result->post_id ]['title'] ) ) {
					$quiz_data[ $result->post_id ]['title'] = get_the_title( $result->post_id );
				}
				if ( ! isset( $quiz_data[ $result->post_id ]['total'] ) ) {
					$quiz_data[ $result->post_id ]['total'] = 0;
				}
				if ( ! isset( $quiz_data[ $result->post_id ]['pass'] ) ) {
					$quiz_data[ $result->post_id ]['pass'] = 0;
				}
				if ( ! isset( $quiz_data[ $result->post_id ]['users'] ) ) {
					$quiz_data[ $result->post_id ]['users'] = array();
				}
				$quiz_data[ $result->post_id ]['total']++;
				$quiz_data[ $result->post_id ]['users'][] = $result->user_id;
				if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
					$quiz_data[ $result->post_id ]['pass']++;
				}
			}
			foreach ( $quiz_data as $quiz_id => $quiz_stat ) {
				if ( 0 == $quiz_data[ $quiz_id ]['total'] ) {
					$quiz_data[ $quiz_id ]['percentage'] = 0;
					continue;
				}
				$quiz_data[ $quiz_id ]['percentage'] = floatval( number_format( 100 * $quiz_data[ $quiz_id ]['pass'] / $quiz_data[ $quiz_id ]['total'], 2, '.', '' ) );// Cast to integer if no decimals.
			}
		}
		return $quiz_data;
	}

	/**
	 * This method is used to return Student-wise quiz activity for a course.
	 *
	 * @param  int $course     Course ID.
	 * @param  int $start_date Timestamp.
	 * @param  int $end_date   Timestamp.
	 * @return Array of Quiz related activity data.
	 */
	public function get_studentwise_activity( $course, $start_date, $end_date ) {
		$quizzes = learndash_course_get_children_of_step( $course, $course, 'sfwd-quiz', 'ids', true );
		if ( empty( $quizzes ) ) {
			return array();
		}
		$quizzes_str         = implode( ', ', $quizzes );
		$activity_query_args = array(
			'post_ids'        => $quizzes_str,
			'activity_type'   => 'quiz',
			'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
			'time_start'      => $start_date,// phpcs:ignore.
			'time_end'        => $end_date,// phpcs:ignore.
			'per_page'        => 1844674407370,
		);
		$quiz_activity       = learndash_reports_get_activity( $activity_query_args );
		$quiz_data           = array();
		if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
			foreach ( $quiz_activity['results'] as $result ) {
				if ( ! isset( $quiz_data[ $result->user_id ]['total'] ) ) {
					$quiz_data[ $result->user_id ]['total'] = 0;
				}
				if ( ! isset( $quiz_data[ $result->user_id ]['pass'] ) ) {
					$quiz_data[ $result->user_id ]['pass'] = 0;
				}
				$quiz_data[ $result->user_id ]['total']++;
				if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
					$quiz_data[ $result->user_id ]['pass']++;
				}
			}
		}
		return $quiz_data;
	}

	/**
	 * This method returns Quiz Statistic info.
	 *
	 * @param  int $course     Course ID.
	 * @param  int $module     Lesson/Topic ID.
	 * @param  int $start_date Timestamp.
	 * @param  int $end_date   Timestamp.
	 * @return Array of WpProQuiz_Model_StatisticRefModel objects.
	 */
	public function get_modulewise_statistics( $course, $module, $start_date, $end_date ) {
		global $wpdb;
		$quizzes      = learndash_course_get_children_of_step( $course, $module, 'sfwd-quiz', 'ids', true );
		$quiz_pro_ids = array_map(
			function( $quiz ) {
				return get_post_meta( $quiz, 'quiz_pro_id', true );
			},
			$quizzes
		);

		$quiz_pro_ids_str = implode( ', ', $quiz_pro_ids );

		$placeholder_args = array();
		$a                = array();
		$args             = array(
			'limit'   => 18446744073709551615,
			'offset'  => 0,
			'order'   => 'DESC',
			'orderby' => 'create_time',
		);

		if ( ( false !== $start_date ) && ( false !== $end_date ) ) {
			$placeholder_args[] = $quiz_pro_ids_str;
			$placeholder_args[] = $start_date;
			$placeholder_args[] = $end_date;
			$placeholder_args[] = $args['orderby'];
			$placeholder_args[] = $args['order'];
			$placeholder_args[] = $args['offset'];
			$placeholder_args[] = $args['limit'];

			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE statref.quiz_id IN (%s) AND statref.create_time >= %s AND statref.create_time <= %s ORDER BY %s %s LIMIT %d, %d', $placeholder_args ), ARRAY_A );

		} elseif ( ( false !== $start_date ) && ( false === $end_date ) ) {
			$placeholder_args[] = $quiz_pro_ids_str;
			$placeholder_args[] = $start_date;
			$placeholder_args[] = $args['orderby'];
			$placeholder_args[] = $args['order'];
			$placeholder_args[] = $args['offset'];
			$placeholder_args[] = $args['limit'];

			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE  statref.quiz_id IN (%s) AND statref.create_time >= %s ORDER BY %s %s LIMIT %d, %d', $placeholder_args ), ARRAY_A );

		} elseif ( ( false === $start_date ) && ( false !== $end_date ) ) {
			$placeholder_args[] = $quiz_pro_ids_str;
			$placeholder_args[] = $end_date;
			$placeholder_args[] = $args['orderby'];
			$placeholder_args[] = $args['order'];
			$placeholder_args[] = $args['offset'];
			$placeholder_args[] = $args['limit'];

			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE  statref.quiz_id IN (%s) AND statref.create_time <= %s ORDER BY %s %s LIMIT %d, %d', $placeholder_args ), ARRAY_A );

		} else {
			$placeholder_args[] = $quiz_pro_ids_str;
			$placeholder_args[] = $args['orderby'];
			$placeholder_args[] = $args['order'];
			$placeholder_args[] = $args['offset'];
			$placeholder_args[] = $args['limit'];

			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( LDLMS_DB::get_table_name( 'quiz_statistic_ref' ) ) . ' as statref WHERE  statref.quiz_id IN (%s) ORDER BY %s %s LIMIT %d, %d', $placeholder_args ), ARRAY_A );

		}
		foreach ( $results as $row ) {
			$a[] = new WpProQuiz_Model_StatisticRefModel( $row );
		}
		return $a;
	}

	/**
	 * This method is used to return the tabular data shown in the Course List Info block.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_course_list_info() {
		$request_data       = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data       = self::get_request_params( $request_data );
		$duration           = self::get_duration_data( $request_data['start_date'], $request_data['end_date'], 'Y-m-d H:i:s' );
		$user_role_access   = self::get_current_user_role_access();
		$accessible_courses = self::get_accessible_courses_for_the_user( get_current_user_id(), $user_role_access, 'course_list_info' );
		$accessible_users   = self::get_accessible_users_for_the_user( get_current_user_id(), $user_role_access, 'course_list_info' );
		$category_query     = array();
		$group_users        = null;//initialize with user ids if the user is group leader & have access to limited users
		global $wpdb;

		$start_date = $duration['start_date'];
		$end_date   = $duration['end_date'];
		if ( empty( $start_date ) || empty( $end_date ) ) {
			$start_date = strtotime( '-1 month' );
			$end_date   = current_time( 'timestamp' );
		}

		if ( isset( $request_data['learner'] ) && ! empty( $request_data['learner'] ) ) {
			$courses = learndash_user_get_enrolled_courses( $request_data['learner'], array(), false );
			if ( isset( $accessible_courses ) ) {
				$courses = array_intersect( $courses, $accessible_courses );
			}
			if ( empty( $courses ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for courses */
						__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'courses' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$table = array();
			foreach ( $courses as $course_id ) {
				$since = ld_course_access_from( $course_id, $request_data['learner'] );
				if ( ! empty( $since ) ) {
					$since = learndash_adjust_date_time_display( $since, 'd M, Y' );
				} else {
					$since = learndash_user_group_enrolled_to_course_from( $request_data['learner'], $course_id );
					if ( ! empty( $since ) ) {
						$since = learndash_adjust_date_time_display( $since, 'd M, Y' );
					}
				}

				$completed = get_user_meta( $request_data['learner'], 'course_completed_' . $course_id, true );
				if ( ! empty( $completed ) ) {
					$completed = learndash_adjust_date_time_display( $completed, 'd M, Y' );
				}
				$quizzes       = learndash_course_get_children_of_step( $course_id, $course_id, 'sfwd-quiz', 'ids', true );
				$quiz_attempts = 0;
				$pass_count    = 0;
				$total_count   = 0;
				if ( ! empty( $quizzes ) ) {
					foreach ( $quizzes as $quiz ) {
						$count         = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(*) FROM ' . esc_sql( \LDLMS_DB::get_table_name( 'user_activity' ) ) . ' WHERE user_id = %d AND post_id = %d AND activity_type = %s AND ( activity_completed >= %d AND activity_completed <= %d ) AND course_id=%d ORDER BY activity_id, activity_started ASC', $request_data['learner'], $quiz, 'quiz', $request_data['start_date'], $request_data['end_date'], $course_id ) );
						$quiz_attempts = $quiz_attempts + $count;
					}
					$quizzes_str         = implode( ', ', $quizzes );
					$activity_query_args = array(
						'post_ids'        => $quizzes_str,
						'user_ids'        => $request_data['learner'],
						'activity_type'   => 'quiz',
						'time_start'      => $start_date,// phpcs:ignore.
						'time_end'        => $end_date,// phpcs:ignore.
						'per_page'        => 1844674407370,
						'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
					);
					$quiz_activity       = learndash_reports_get_activity( $activity_query_args );
					$pass_count          = 0;
					$total_count         = 0;
					if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
						foreach ( $quiz_activity['results'] as $result ) {
							$total_count++;
							if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
								$pass_count++;
							}
						}
					}
				}
				$progress   = learndash_user_get_course_progress( $request_data['learner'], $course_id, 'summary' );
				$percentage = 100;
				if ( 0 < $progress['total'] ) {
					$percentage = floatval( number_format( 100 * $progress['completed'] / $progress['total'], 2, '.', '' ) );// Cast to integer if no decimals.
				}
				$table[] = array(
					'course'          => get_the_title( $course_id ),
					'start_date'      => ! empty( $since ) ? $since : '-',
					'end_date'        => ! empty( $completed ) ? $completed : '-',
					'quiz_count'      => count( $quizzes ),
					'quiz_attempts'   => $quiz_attempts,
					'pass_count'      => $pass_count,
					'fail_count'      => $total_count - $pass_count,
					'course_progress' => $percentage,
				);
			}
			return new WP_REST_Response(
				array(
					'requestData' => self::get_values_for_request_params( $request_data ),
					'table'       => $table,
				),
				200
			);
		}

		$data = array();

		if ( isset( $request_data['course'] ) && ! empty( $request_data['course'] ) ) {
			if ( isset( $accessible_courses ) ) {
				if ( ! in_array( $request_data['course'], $accessible_courses ) ) {
					return new WP_Error(
						'unauthorized',
						sprintf(/* translators: %s: custom label for course */
							__( 'You don\'t have access to this %s.', 'learndash-reports-by-wisdmlabs' ),
							\LearnDash_Custom_Label::label_to_lower( 'course' )
						),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					);
				}
			}
		}

		if ( ! empty( $request_data['topic'] ) ) {
			$students      = learndash_get_users_for_course( $request_data['course'], array(), false );
			$students      = ! is_array( $students ) ? $students->get_results() : $students;
			$students      = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			$student_count = is_array( $students ) ? count( $students ) : $students->get_total();
			if ( empty( $students ) ) {
				return new WP_Error(
					'no-data',
					__( 'No Data to display', 'learndash-reports-by-wisdmlabs' ),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$table         = array();
			$quizzes       = learndash_course_get_children_of_step( $request_data['course'], $request_data['topic'], 'sfwd-quiz', 'ids', true );
			$quiz_count    = count( $quizzes );
			$quiz_activity = array();
			foreach ( $students as $student_id ) {
				$student_progress       = learndash_user_get_course_progress( $student_id, $request_data['course'], 'activity' );
				$student_progress_steps = learndash_user_get_course_progress( $student_id, $request_data['course'], 'summary' );
				$student_info           = get_userdata( $student_id );
				$attempts               = 0;
				$pass                   = 0;
				$score                  = 0;
				$counter                = 0;
				if ( $quiz_count > 0 ) {
					$quizzes_str         = implode( ', ', $quizzes );
					$activity_query_args = array(
						'post_ids'        => $quizzes_str,
						'course_ids'      => $request_data['course'],
						'user_ids'        => $student_id,
						'activity_type'   => 'quiz',
						'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
						'per_page'        => 1844674407370,
					);
					$quiz_activity       = learndash_reports_get_activity( $activity_query_args );
					if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
						foreach ( $quiz_activity['results'] as $result ) {
							$attempts++;
							if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
								$pass++;
							}
							if ( ( isset( $result->activity_meta['percentage'] ) ) && $result->activity_meta['percentage'] ) {
								$counter++;
								$score += (float) $result->activity_meta['percentage'];
							}
						}
					}
					if ( $counter > 0 ) {
						$score /= $counter;
					}
				}
				foreach ( $student_progress as $key => $progress ) {
					if ( empty( $progress ) || 'sfwd-topic:' . $request_data['topic'] != $key ) {
						continue;
					}
					$entry   = array(
						'name'      => $student_info->display_name,
						'email'     => $student_info->user_email,
						'status'    => $student_progress[ 'sfwd-topic:' . $request_data['topic'] ]['activity_status'] ? __( 'Complete', 'learndash-reports-by-wisdmlabs' ) : __( 'Not Complete', 'learndash-reports-by-wisdmlabs' ),
						/* translators: %1$d: Steps Completed, %2$d: Total Steps */
						'steps'     => sprintf( __( '%1$d of %2$d', 'learndash-reports-by-wisdmlabs' ), $student_progress_steps['completed'], $student_progress_steps['total'] ),
						'date'      => date_i18n( 'd M, Y', $student_progress[ 'sfwd-topic:' . $request_data['topic'] ]['activity_completed'] ),
						'time'      => date_i18n( 'H:i:s', (int) $student_progress[ 'sfwd-topic:' . $request_data['topic'] ]['activity_completed'] - (int) $student_progress[ 'sfwd-topic:' . $request_data['topic'] ]['activity_started'] ),
						'attempts'  => $attempts,
						'pass_rate' => empty( $attempts ) ? '-' : floatval( number_format( 100 * $pass / $attempts, 2, '.', '' ) ) . '%',
						'avg_score' => floatval( number_format( $score, 2, '.', '' ) ) . '%',
					);
					$table[] = $entry;
					break;
				}
			}
			return new WP_REST_Response(
				array(
					'requestData' => self::get_values_for_request_params( $request_data ),
					'table'       => $table,
				),
				200
			);
		}

		if ( ! empty( $request_data['lesson'] ) ) {
			$students      = learndash_get_users_for_course( $request_data['course'], array(), false );
			$students      = ! is_array( $students ) ? $students->get_results() : $students;
			$students      = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			$student_count = is_array( $students ) ? count( $students ) : $students->get_total();
			if ( empty( $students ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for lesson */
						__( 'No %1$s assigned to this %2$s', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'topics' ),
						\LearnDash_Custom_Label::label_to_lower( 'lesson' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$table         = array();
			$quizzes       = learndash_course_get_children_of_step( $request_data['course'], $request_data['lesson'], 'sfwd-quiz', 'ids', true );
			$quiz_count    = count( $quizzes );
			$quiz_activity = array();
			foreach ( $students as $student_id ) {
				$student_progress       = learndash_user_get_course_progress( $student_id, $request_data['course'], 'activity' );
				$student_progress_steps = learndash_user_get_course_progress( $student_id, $request_data['course'], 'summary' );
				$student_info           = get_userdata( $student_id );
				$attempts               = 0;
				$pass                   = 0;
				$score                  = 0;
				$counter                = 0;
				if ( $quiz_count > 0 ) {
					$quizzes_str         = implode( ', ', $quizzes );
					$activity_query_args = array(
						'post_ids'        => $quizzes_str,
						'course_ids'      => $request_data['course'],
						'user_ids'        => $student_id,
						'activity_type'   => 'quiz',
						'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
						'per_page'        => 1844674407370,
					);

					$quiz_activity = learndash_reports_get_activity( $activity_query_args );
					if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
						foreach ( $quiz_activity['results'] as $result ) {
							$attempts++;
							if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
								$pass++;
							}
							if ( ( isset( $result->activity_meta['percentage'] ) ) && $result->activity_meta['percentage'] ) {
								$counter++;
								$score += (float) $result->activity_meta['percentage'];
							}
						}
					}
					if ( $counter > 0 ) {
						$score /= $counter;
					}
				}
				foreach ( $student_progress as $key => $progress ) {
					if ( empty( $progress ) || 'sfwd-lessons:' . $request_data['lesson'] != $key ) {
						continue;
					}
					$entry = array(
						'name'            => $student_info->display_name,
						'email'           => $student_info->user_email,
						'status'          => $student_progress[ 'sfwd-lessons:' . $request_data['lesson'] ]['activity_status'] ? __( 'Complete', 'learndash-reports-by-wisdmlabs' ) : __( 'Not Complete', 'learndash-reports-by-wisdmlabs' ),
						/* translators: %1$d: Steps Completed, %2$d: Total Steps */
						'steps'           => sprintf( __( '%1$d of %2$d', 'learndash-reports-by-wisdmlabs' ), $student_progress_steps['completed'], $student_progress_steps['total'] ),
						'completion_rate' => ! empty( $student_progress_steps['total'] ) ? floatval( number_format( 100 * $student_progress_steps['completed'] / $student_progress_steps['total'], 2, '.', '' ) ) . '%' : '100%',
						'date'            => date_i18n( 'd M, Y', $student_progress[ 'sfwd-lessons:' . $request_data['lesson'] ]['activity_completed'] ),
						'time'            => date_i18n( 'H:i:s', (int) $student_progress[ 'sfwd-lessons:' . $request_data['lesson'] ]['activity_completed'] - (int) $student_progress[ 'sfwd-lessons:' . $request_data['lesson'] ]['activity_started'] ),
						'attempts'        => $attempts,
						'pass_rate'       => empty( $attempts ) ? '-' : floatval( number_format( 100 * $pass / $attempts, 2, '.', '' ) ) . '%',
						'avg_score'       => empty( $score ) ? '-' : floatval( number_format( $score, 2, '.', '' ) ) . '%',
					);

					$table[] = $entry;

					break;
				}
			}

			return new WP_REST_Response(
				array(
					'requestData' => self::get_values_for_request_params( $request_data ),
					'table'       => $table,
				),
				200
			);
		}

		if ( ! empty( $request_data['course'] ) ) {
			$students      = learndash_get_users_for_course( $request_data['course'], array(), false );
			$students      = ! is_array( $students ) ? $students->get_results() : $students;
			$students      = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			$student_count = is_array( $students ) ? count( $students ) : $students->get_total();

			if ( empty( $students ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for course */
						__( 'No Students enrolled in this %s.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::label_to_lower( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$table       = array();
			$quizzes     = learndash_course_get_children_of_step( $request_data['course'], $request_data['course'], 'sfwd-quiz', 'ids', true );
			$quizzes_str = implode( ', ', $quizzes );
			$quiz_count  = count( $quizzes );
			foreach ( $students as $student_id ) {
				$attempts = 0;
				$pass     = 0;
				$score    = 0;
				$counter  = 0;
				if ( 0 < $quiz_count ) {
					$activity_query_args = array(
						'post_ids'        => $quizzes_str,
						'course_ids'      => $request_data['course'],
						'user_ids'        => $student_id,
						'activity_type'   => 'quiz',
						'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
						'per_page'        => 1844674407370,
					);
					$quiz_activity       = learndash_reports_get_activity( $activity_query_args );
					if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
						foreach ( $quiz_activity['results'] as $result ) {
							$attempts++;
							if ( ( isset( $result->activity_meta['pass'] ) ) && $result->activity_meta['pass'] ) {
								$pass++;
							}
							if ( ( isset( $result->activity_meta['percentage'] ) ) && $result->activity_meta['percentage'] ) {
								$counter++;
								$score += (float) $result->activity_meta['percentage'];
							}
						}
					}
					if ( $counter > 0 ) {
						$score /= $counter;
					}
				}
				$progress = learndash_user_get_course_progress( $student_id, $request_data['course'], 'summary' );
				// $student_progress = learndash_user_get_course_progress( $student_id, $request_data['course'], 'activity' );
				$student_info = get_userdata( $student_id );
				$percentage   = 100;
				/*if ( $student_progress[ 'sfwd-courses:' . $request_data['course'] ]['activity_status'] ) {
					$status = __( 'Completed', 'learndash-reports-by-wisdmlabs' );
				} elseif ( empty( $student_progress[ 'sfwd-courses:' . $request_data['course'] ]['activity_updated'] ) ) {
					$status = __( 'Not Started', 'learndash-reports-by-wisdmlabs' );
				} else {
					$status = __( 'In Progress', 'learndash-reports-by-wisdmlabs' );
				}*/
				if ( 0 != $progress['total'] ) {
					$percentage = floatval( number_format( 100 * $progress['completed'] / $progress['total'], 2, '.', '' ) );// Cast to integer if no decimals.
				}

				$since = ld_course_access_from( $request_data['course'], $student_id );
				if ( ! empty( $since ) ) {
					$since = learndash_adjust_date_time_display( $since, 'd M, Y' );
				} else {
					$since = learndash_user_group_enrolled_to_course_from( $student_id, $request_data['course'] );
					if ( ! empty( $since ) ) {
						$since = learndash_adjust_date_time_display( $since, 'd M, Y' );
					}
				}

				$completed = get_user_meta( $student_id, 'course_completed_' . $request_data['course'], true );
				if ( ! empty( $completed ) ) {
					$completed = learndash_adjust_date_time_display( $completed, 'd M, Y' );
				}

				$table[] = array(
					'name'            => $student_info->display_name,
					'email'           => $student_info->user_email,
					// 'status'		  => $status,
					'started'         => ! empty( $since ) ? $since : '-',
					'course_progress' => $percentage . '%',
					'completed'       => ! empty( $completed ) ? $completed : '-',
					'time_spent'      => date_i18n( 'H:i:s', $this->learndash_get_user_course_attempts_time_spent( $student_id, $request_data['course'] ) ),
					'quiz_attempts'   => $attempts,
					'pass_rate'       => empty( $attempts ) ? '-' : floatval( number_format( 100 * $pass / $attempts, 2, '.', '' ) ) . '%',
					'avg_score'       => empty( $score ) ? '-' : floatval( number_format( $score, 2, '.', '' ) ) . '%',
				);
			}
			return new WP_REST_Response(
				array(
					'requestData' => self::get_values_for_request_params( $request_data ),
					'table'       => $table,
				),
				200
			);
		}

		$args         = array(
			'post_type'      => 'sfwd-courses',
			'posts_per_page' => '-1',
			'post__in'       => $accessible_courses,
		);
		$category_str = '-';

		// Now consider the scenario where a category is selected. (Pro feature).
		if ( isset( $request_data['category'] ) && ! empty( $request_data['category'] ) ) {
			// Check if course category enabled.
			if ( ! taxonomy_exists( 'ld_course_category' ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category disabled. Please contact admin.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			// Check if valid category passed.
			if ( ! is_object( get_term_by( 'id', $request_data['category'], 'ld_course_category' ) ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}
			$args['tax_query'] = array(
				array(
					'taxonomy'         => 'ld_course_category',
					'field'            => 'term_id',
					'terms'            => $request_data['category'], // Where term_id of Term 1 is "1".
					'include_children' => false,
				),
			);
			$category_str      = get_term( $request_data['category'] )->name;
		}
		if ( isset( $request_data['group'] ) && ! empty( $request_data['group'] ) ) {
			$args['meta_query'] = array(
				array(
					'key'     => 'learndash_group_enrolled_' . $request_data['group'],
					'compare' => 'EXISTS',
				),
			);
		}
		$courses = get_posts( $args );
		// Check if any courses present in the category.
		if ( empty( $courses ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		$table        = array();
		$course_count = count( $courses );
		$user         = wp_get_current_user();
		foreach ( $courses as $course ) {
			if ( '-' == $category_str ) {
				$categories = wp_get_object_terms( $course->ID, 'ld_course_category' );
				if ( ! empty( $categories ) && ! is_wp_error( $categories ) ) {
					$categories   = wp_list_pluck( $categories, 'name' );
					$category_str = implode( ', ', $categories );
				}
			}

			$groups_str = '-';
			$group_ids  = learndash_get_course_groups( $course->ID );
			if ( ! current_user_can( 'manage_options' ) ) {
				if ( in_array( 'group_leader', (array) $user->roles, true ) ) {
					$associated_groups = learndash_get_administrators_group_ids( get_current_user_id() );
					foreach ( $group_ids as $key => $group ) {
						if ( ! in_array( $group, $associated_groups ) ) {
							unset( $group_ids[ $key ] );
						}
					}
				}
			}
			if ( ! empty( $group_ids ) ) {
				$groups     = array_map(
					function( $group_id ) {
						return str_replace( '&#8211;', '-', get_the_title( $group_id ) );
					},
					$group_ids
				);
				$groups_str = implode( ', ', $groups );
			}

			if ( function_exists( 'ir_get_instructor_complete_course_list' ) ) {
				// Get list of co-instructors.
				$co_instructor_list = get_post_meta( $course->ID, 'ir_shared_instructor_ids', 1 );
				$all_instructor_ids = explode( ',', $co_instructor_list );

				// Include course author.
				array_unshift( $all_instructor_ids, $course->post_author );

				// Remove any duplicates.
				$all_instructor_ids = array_filter( array_unique( $all_instructor_ids ) );
				$instructors        = array_map(
					function( $instructor_id ) {
						return get_userdata( $instructor_id )->display_name;
					},
					$all_instructor_ids
				);
				$instructor         = implode( ', ', $instructors );
			} else {
				$instructor = get_userdata( $course->post_author )->display_name;
			}

			$students      = learndash_get_users_for_course( $course->ID, array(), false );
			$students      = ! is_array( $students ) ? $students->get_results() : $students;
			$students      = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			$student_count = is_array( $students ) ? count( $students ) : $students->get_total();

			$quizzes           = learndash_course_get_children_of_step( $course->ID, $course->ID, 'sfwd-quiz', 'ids', true );
			$quiz_count        = count( $quizzes );
			$time              = 0;
			$quiz_time         = 0;
			$quiz_average_time = 0;
			$not_started_count = 0;
			$in_progress_count = 0;
			$completed_count   = 0;
			$score             = 0;
			$counter           = 0;
			if ( 0 < $quiz_count ) {
				$quizzes_str         = implode( ', ', $quizzes );
				$activity_query_args = array(
					'post_ids'        => $quizzes_str,
					'course_ids'      => $course->ID,
					'activity_type'   => 'quiz',
					'activity_status' => array( 'NOT_STARTED', 'IN_PROGRESS', 'COMPLETED' ),
					'per_page'        => 1844674407370,
				);
				$quiz_activity       = learndash_reports_get_activity( $activity_query_args );
				if ( ( isset( $quiz_activity['results'] ) ) && ( ! empty( $quiz_activity['results'] ) ) ) {
					foreach ( $quiz_activity['results'] as $result ) {
						if ( ( isset( $result->activity_meta['percentage'] ) ) && $result->activity_meta['percentage'] ) {
							$counter++;
							$score += (float) $result->activity_meta['percentage'];
						}
					}
				}
				if ( $counter > 0 ) {
					$score /= $counter;
				}
			}
			if ( $student_count > 0 ) {
				foreach ( $students as $student_id ) {
					$time   = $time + $this->learndash_get_user_course_attempts_time_spent( $student_id, $course->ID );
					$status = learndash_user_get_course_progress( $student_id, $course->ID, 'summary' );
					switch ( $status['status'] ) {
						case 'in_progress':
							$in_progress_count++;
							break;
						case 'completed':
							$completed_count++;
							break;
						case 'not_started':
						default:
							$not_started_count++;
							break;
					}
				}
			}
			if ( ! empty( $quizzes ) && 0 < $student_count ) {
				foreach ( $students as $student_id ) {
					foreach ( $quizzes as $quiz ) {
						$quiz_time = $quiz_time + $this->learndash_get_user_quiz_attempts_time_spent( $student_id, $quiz );
					}
				}
				$quiz_average_time = floatval( number_format( $quiz_time / $student_count, 2, '.', '' ) );// Cast to integer if no decimals.
			}
			$table_data = array(
				'course' => $course->post_title,
			);
			if ( ! isset( $request_data['category'] ) || empty( $request_data['category'] ) ) {
				$table_data['category'] = $category_str;
				$category_str           = '-';
			}
			$table_data = array_merge(
				$table_data,
				array(
					/* translators: %1$d: Completed Count %2$d: Total Student count */
					'completed_users'  => sprintf( __( '%1$d of %2$d', 'learndash-reports-by-wisdmlabs' ), $completed_count, $student_count ),
					'in_progress'      => $in_progress_count,
					'not_started'      => $not_started_count,
					'completion_rate2' => empty( $student_count ) ? '-' : floatval( number_format( 100 * $completed_count / $student_count, 2, '.', '' ) ) . '%',
					'total_time_spent' => date_i18n( 'H:i:s', $time ),
					'avg_score'        => floatval( number_format( $score, 2, '.', '' ) ) . '%',
					'quizzes'          => $quiz_count,
					'quiz_time'        => date_i18n( 'H:i:s', $quiz_average_time ),
					'groups'           => $groups_str,
					'instructors'      => $instructor,
				)
			);
			$table[]    = $table_data;
		}
		if ( $course_count <= 0 ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		return new WP_REST_Response(
			array(
				'requestData' => self::get_values_for_request_params( $request_data ),
				'table'       => $table,
			),
			200
		);
	}


	/**
	 * This method is used to calculate course completion rate across courses by all the students.
	 * This method is doesn't support the date-wise filters because the completion dates aren't stored in the database.
	 *
	 * @return WP_REST_Response/WP_Error Objects.
	 */
	public function get_course_completion_rate() {
		// Get Inputs.
		$request_data = filter_input_array( INPUT_GET, FILTER_SANITIZE_STRING );
		$request_data = self::get_request_params( $request_data );
		unset( $request_data['start_date'] );
		unset( $request_data['end_date'] );
		$user_role_access   = self::get_current_user_role_access();
		$accessible_courses = self::get_accessible_courses_for_the_user( get_current_user_id(), $user_role_access, 'course_completion_rate' );
		$accessible_users   = self::get_accessible_users_for_the_user( get_current_user_id(), $user_role_access, 'course_completion_rate' );
		$category_query     = array();
		$group_users        = null;//initialize with user ids if the user is group leader & have access to limited users

		if ( isset( $request_data['learner'] ) && ! empty( $request_data['learner'] ) ) {
			$learner_courses    = learndash_user_get_enrolled_courses( $request_data['learner'], array(), false );
			$accessible_courses = null != $accessible_courses ? array_intersect( $accessible_courses, $learner_courses ) : $learner_courses;
			$accessible_users   = null != $accessible_users ? array_intersect( $accessible_users, array( $request_data['learner'] ) ) : $request_data['learner'];

			if ( empty( $accessible_courses ) ) {
				return new WP_Error(
					'no-data',
					sprintf(/* translators: %s: custom label for courses */
						__( 'No accessible %s found', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'courses' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}

			$total_percentage  = 0;
			$progress_data     = array();
			$completed_count   = 0;
			$not_started_count = 0;
			$in_progress_count = 0;

			foreach ( $accessible_courses as $course_id ) {
				$progress = learndash_user_get_course_progress( $request_data['learner'], $course_id, 'summary' );
				if ( empty( $progress ) || ! isset( $progress['total'] ) || 0 == $progress['total'] ) {
					$percentage = 0;
					$not_started_count++;
				} else {
					$percentage = floatval( number_format( 100 * $progress['completed'] / $progress['total'], 2, '.', '' ) );// Cast to integer if no decimals.
					if ( 100 == $percentage ) {
						$completed_count++;
					} elseif ( 0 == $percentage ) {
						$not_started_count++;
					} else {
						$in_progress_count++;
					}
				}
				$progress_data[ $course_id ] = array(
					'user_name' => get_the_title( $course_id ),
					'progress'  => 100 <= $percentage ? $percentage : 0,
				);
				$total_percentage            = 100 <= $percentage ? $total_percentage + $percentage : $total_percentage;
			}

			// Calculate average across courses.
			$average_completion_percentage = $total_percentage / count( $accessible_courses );
			return new WP_REST_Response(
				array(
					'requestData'             => self::get_values_for_request_params( $request_data ),
					'progress_data_new'       => $progress_data,
					'averageCourseCompletion' => $average_completion_percentage,
					'completedCount'          => $completed_count,
					'notstartedCount'         => $not_started_count,
					'inprogressCount'         => $in_progress_count,
				),
				200
			);
		} elseif ( isset( $request_data['course'] ) && ! empty( $request_data['course'] ) ) {
			if ( isset( $accessible_courses ) ) {
				if ( ! in_array( $request_data['course'], $accessible_courses ) ) {
					return new WP_Error(
						'unauthorized',
						sprintf(/* translators: %s: custom label for course */
							__( 'You don\'t have access to this %s.', 'learndash-reports-by-wisdmlabs' ),
							\LearnDash_Custom_Label::label_to_lower( 'course' )
						),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					);
				}
			}
			$course = get_post( $request_data['course'] );
			// Check for valid course.
			if ( empty( $course ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' )
					),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}

			$course_users     = learndash_get_users_for_course( $request_data['course'], array(), false );
			$course_users     = is_array( $course_users ) ? $course_users : $course_users->get_results();
			$accessible_users = null != $accessible_users ? array_intersect( $course_users, $accessible_users ) : $course_users;
			$student_count    = count( $accessible_users );

			// Check if any students enrolled.
			if ( empty( $accessible_users ) ) {
				return new WP_Error(
					'no-data',
					__( 'No Students enrolled', 'learndash-reports-by-wisdmlabs' ),
					array( 'requestData' => self::get_values_for_request_params( $request_data ) )
				);
			}

			$completed_count   = 0;
			$not_started_count = 0;
			$in_progress_count = 0;
			$total_percentage  = 0;
			// Get progress for each student.
			foreach ( $accessible_users as $student ) {

				$progress = learndash_user_get_course_progress( $student, $course->ID, 'summary' );
				if ( 0 < $progress['total'] ) {
					$percentage = floatval( number_format( 100 * $progress['completed'] / $progress['total'], 2, '.', '' ) );// Cast to integer if no decimals.
				} else {
					$percentage = 0;
				}
				$progress_data[ $student ] = array(
					'user_name' => get_userdata( $student )->display_name,
					'progress'  => 100 <= $percentage ? $percentage : 0,
				);
				if ( 0 == $percentage ) {
					$not_started_count++;
				} elseif ( 100 == $percentage ) {
					$completed_count++;
				} else {
					$in_progress_count++;
				}
				$total_percentage = 100 <= $percentage ? $total_percentage + $percentage : $total_percentage;
			}
			// Calculate average across students.
			$average_completion_percentage = $total_percentage / $student_count;
			return new WP_REST_Response(
				array(
					'requestData'             => self::get_values_for_request_params( $request_data ),
					'progress_data'           => $progress_data,
					'averageCourseCompletion' => $average_completion_percentage,
					'completedCount'          => $completed_count,
					'notstartedCount'         => $not_started_count,
					'inprogressCount'         => $in_progress_count,
				),
				200
			);
		} elseif ( isset( $request_data['category'] ) && ! empty( $request_data['category'] ) ) {
			// Check if course category enabled.
			if ( ! taxonomy_exists( 'ld_course_category' ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category disabled. Please contact admin.', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' ),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					)
				);
			}
			// Check if valid category passed.
			if ( ! is_object( get_term_by( 'id', $request_data['category'], 'ld_course_category' ) ) ) {
				return new WP_Error(
					'invalid-input',
					sprintf(/* translators: %s: custom label for course */
						__( '%s Category doesn\'t exist', 'learndash-reports-by-wisdmlabs' ),
						\LearnDash_Custom_Label::get_label( 'course' ),
						array( 'requestData' => self::get_values_for_request_params( $request_data ) )
					)
				);
			}
			$category_query = array(
				array(
					'taxonomy'         => 'ld_course_category',
					'field'            => 'term_id',
					'terms'            => $request_data['category'], // Where term_id of Term 1 is "1".
					'include_children' => false,
				),
			);
		} elseif ( isset( $request_data['group'] ) && ! empty( $request_data['group'] ) ) {
			$group_courses      = learndash_group_enrolled_courses( $request_data['group'] );
			$accessible_courses = null != $accessible_courses ? array_intersect( $group_courses, $accessible_courses ) : $group_courses;
			$group_users        = self::get_ld_group_user_ids( $request_data['group'] );
			$accessible_users   = null != $accessible_users ? array_intersect( $group_users, $accessible_users ) : $group_users;
		}

		$args = array(
			'post_type'      => 'sfwd-courses',
			'posts_per_page' => '-1',
			'post__in'       => $accessible_courses,
		);

		if ( ! empty( $category_query ) ) {
			$args['tax_query'] = $category_query;
		}

		$courses     = get_posts( $args );
		$course_data = array();
		// Check if any courses present in the category.
		// Check if any courses present in the category.
		if ( empty( $courses ) || ( null != $accessible_courses && empty( $accessible_courses ) ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}
		// Average Completion percentage count variables.
		$course_count     = count( $courses );
		$total_completion = 0;

		// Count for students based on completion status.
		$not_started_count = 0;
		$completed_count   = 0;
		$in_progress_count = 0;
		foreach ( $courses as $course ) {
			$students   = learndash_get_users_for_course( $course->ID, array(), false ); // Third argument is $exclude_admin.
			$students   = is_array( $students ) ? $students : $students->get_results();
			$students   = null != $accessible_users ? array_intersect( $accessible_users, $students ) : $students;
			$class_size = is_array( $students ) ? count( $students ) : $students->get_total();
			$percentage = 0;

			// If no students in the course then the course has 0 percent completion.
			if ( empty( $students ) ) {
				$course_data[ $course->ID ] = array(
					'completion' => $percentage,
					'title'      => get_the_title( $course->ID ),
				);
				continue;
			}

			foreach ( $students as $student ) {
				// Get course progress info.
				$progress = learndash_user_get_course_progress( $student, $course->ID, 'summary' );
				if ( ! isset( $progress['total'] ) || empty( $progress['total'] ) ) {
					$percentage = $percentage + 0;
					continue;
				}
				// Logic to calculate the status-wise count.
				switch ( $progress['status'] ) {
					case 'completed':
						$completed_count++;
						$percentage = $percentage + ( ( 100 * $progress['completed'] ) / $progress['total'] );
						break;
					case 'not_started':
						$not_started_count++;
						break;
					case 'in_progress':
						$in_progress_count++;
						break;
					default:
						break;
				}
			}
			// Average completion Course-wise.
			$course_data[ $course->ID ] = array(
				'completion' => floatval( number_format( $percentage / $class_size, 2, '.', '' ) ),
				'title'      => get_the_title( $course->ID ),
			);

			$total_completion = $total_completion + $course_data[ $course->ID ]['completion'];
		}

		if ( $course_count <= 0 ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				),
				array( 'requestData' => self::get_values_for_request_params( $request_data ) )
			);
		}

		// Average Completion across all courses.
		$overall_average_completion = floatval( number_format( $total_completion / $course_count, 2, '.', '' ) );// Cast to integer if no decimals.

		return new WP_REST_Response(
			array(
				'requestData'             => self::get_values_for_request_params( $request_data ),
				'averageCourseCompletion' => $overall_average_completion,
				'courseWiseCompletion'    => $course_data,
				'completedCount'          => $completed_count,
				'notstartedCount'         => $not_started_count,
				'inprogressCount'         => $in_progress_count,
			),
			200
		);
	}
}
