<?php
/**
 * This file contains a class that is used to setup the LearnDash endpoints.
 *
 * @package learndash-reports-by-wisdmlabs
 */

/**
 * Class that sets up all the LearnDash endpoints
 *
 * @author     WisdmLabs
 * @since      1.0.0
 * @subpackage LearnDash API
 */
class WRLD_LearnDash_Endpoints {

	/**
	 * This static contains the number of points being assigned on course completion
	 *
	 * @var    Instance of WRLD_LearnDash_Endpoints class
	 * @since  1.0.0
	 * @access private
	 */
	private static $instance = null;

	/**
	 * API version
	 *
	 * @var string
	 */

	private static $version = 'v1';
	/**
	 * API namespace.
	 *
	 * @var string
	 */

	private static $namespace = 'rp';

	/**
	 * This static method is used to return a single instance of the class
	 *
	 * @since  1.0.0
	 * @access public
	 * @return Object
	 */
	public static function get_instance() {
		if ( empty( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * This getter method returns the API version.
	 *
	 * @return string
	 */
	public static function get_api_version() {
		return self::$version;
	}

	/**
	 * This is a constructor which will be used to initialize required hooks.
	 *
	 * @since  1.0.0
	 * @access private
	 * @see    register_hook static method.
	 */
	private function __construct() {
		$this->include_api_files();
		// Set API endpoint version.
		self::register_hook(
			'init',
			'set_endpoint_version',
			$this,
			array(
				'type'     => 'action',
				'priority' => 100,
				'num_args' => 0,
			)
		);
		// Register custom endpoints.
		self::register_hook(
			'rest_api_init',
			'register_custom_endpoints',
			$this,
			array(
				'type'     => 'action',
				'priority' => 10,
				'num_args' => 0,
			)
		);
	}

	/**
	 * This method is used to set endpoint version and namespace.
	 */
	public function set_endpoint_version() {
		if ( ! defined( 'LEARNDASH_VERSION' ) ) {
			return;
		}
		self::$version   = 'v1';
		self::$namespace = 'rp';
	}

	/**
	 * This method includes all the classes related to our endpoints.
	 */
	public function include_api_files() {
		include_once WRLD_REPORTS_PATH . '/includes/apis/class-wrld-revenue-api.php';
		include_once WRLD_REPORTS_PATH . '/includes/apis/class-wrld-course-progress-info.php';
		include_once WRLD_REPORTS_PATH . '/includes/apis/class-wrld-course-time-tracking.php';
		include_once WRLD_REPORTS_PATH . '/includes/apis/class-wrld-quiz-reporting-tools.php';
		include_once WRLD_REPORTS_PATH . '/includes/apis/class-wrld-quiz-export-db.php';

		WRLD_Course_Time_Tracking::get_instance()->init_hooks();
	}

	/**
	 * Add all the WordPress actions/filters
	 *
	 * @since  1.0.0
	 * @access private
	 * @param  [string] $hook     [action-hook].
	 * @param  [string] $callback [class method].
	 * @param  [object] $scope    [class name or object instance].
	 * @param  [array]  $args     [type of hook, it's priority and number of arguments to the callback].
	 */
	private static function register_hook( $hook, $callback, $scope, $args ) {
		call_user_func_array( 'add_' . $args['type'], array( $hook, array( $scope, $callback ), $args['priority'], $args['num_args'] ) );
	}

	/**
	 * This is a method used to register the My Courses Endpoint
	 *
	 * @since  1.0.0
	 * @access public
	 */
	public function register_custom_endpoints() {
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/total-revenue-earned/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Revenue_API::get_instance(),
					'get_total_revenue',
				),
				'permission_callback' => array( 'WRLD_LearnDash_Endpoints', 'total_revenue_data_permission_callback' ),
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/total-courses/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Revenue_API::get_instance(),
					'get_total_courses',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/total-learners/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Revenue_API::get_instance(),
					'get_total_learners',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/revenue-from-courses/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Revenue_API::get_instance(),
					'get_coursewise_revenue',
				),
				'permission_callback' => array( 'WRLD_LearnDash_Endpoints', 'coursewise_revenue_data_permission_callback' ),
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/daily-enrollments/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_daily_enrollments',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/time-spent-on-a-course/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Time_Tracking::get_instance(),
					'get_course_time_spent',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/course-completion-rate/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_course_completion_rate',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/quiz-completion-time-per-course/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Time_Tracking::get_instance(),
					'get_quiz_completion_time',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/quiz-completion-rate/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_quiz_completion_rate',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/learner-pass-fail-rate-per-course/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_quiz_passing_rate',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/average-quiz-attempts/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_average_quiz_attempts',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/course-list-info/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_course_list_info',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/qre-live-search/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Quiz_Reporting_Tools::get_instance(),
					'qre_live_search_results',
				),
				'permission_callback' => '__return_true',
			)
		);
		register_rest_route(
			self::$namespace . '/' . self::$version,
			'/pending-assignments/',
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array(
					WRLD_Course_Progress_Info::get_instance(),
					'get_pending_assignments_info',
				),
				'permission_callback' => '__return_true',
			)
		);
	}


	/**
	 * Determines wether the user have proper permission to access the data from the api callback.
	 *
	 * @return bool true when the data is accesible falce when data is not accessible.
	 */
	public static function total_revenue_data_permission_callback() {
		return apply_filters( 'wrld_filter_total_revenue_access_permission', '__return_true' );
	}

	/**
	 * Determines wether the user have proper permission to access the data from the api callback.
	 *
	 * @return bool true when the data is accesible falce when data is not accessible.
	 */
	public static function coursewise_revenue_data_permission_callback() {
		return apply_filters( 'wrld_filter_coursewise_revenue_access_permission', '__return_true' );
	}
}
