<?php
/**
 * This file contains a class that is used to setup the LearnDash endpoints
 *
 * @package learndash-reports-by-wisdmlabs
 */

/**
 * Class that sets up all the LearnDash endpoints
 *
 * @since      1.0.0
 */
class WRLD_Quiz_Reporting_Tools {

	/**
	 * This static contains the number of points being assigned on course completion
	 *
	 * @var    Instance of WRLD_Quiz_Reporting_Tools class
	 * @since  1.0.0
	 * @access private
	 */
	private static $instance = null;

	/**
	 * This static method is used to return a single instance of the class
	 *
	 * @since  1.0.0
	 * @access public
	 * @return Object
	 */
	public static function get_instance() {
		if ( empty( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * This method is a callback to the API endpint, 'qre-live-search', which searches the term
	 * entered in students & quizes & returns the results
	 *
	 * @return WP_REST_Response
	 */
	public function qre_live_search_results() {
		// Get Inputs.
		$search_term = sanitize_text_field( ! empty( $_GET['search_term'] ) ? $_GET['search_term'] : '' );// WPCS: sanitization ok, CSRF ok.

		if ( empty( $search_term ) ) {
			return new WP_Error(
				'no-data',
				sprintf(/* translators: %s: custom label for courses */
					__( 'No %s found', 'learndash-reports-by-wisdmlabs' ),
					\LearnDash_Custom_Label::get_label( 'courses' )
				)
			);
		}
		if ( strlen( $search_term ) < 3 ) {
			return new WP_Error( 'query-length', __( 'Search term is short enter atleast 3 letters', 'learndash-reports-by-wisdmlabs' ) );
		}

		$results        = array();
		$user_results   = array();
		$quiz_results   = self::qre_search_quizzes( $search_term );
		$user_results   = self::qre_search_users( $search_term );
		$course_results = self::qre_search_courses( $search_term );
		$results        = array_merge( $user_results, $quiz_results );
		$results        = array_merge( $results, $course_results );

		return new WP_REST_Response(
			array(
				'search_results' => $results,
			),
			200
		);
	}


	/**
	 * Function is used to get Courses that match the queried string.
	 *
	 * @param  string  $query Search Query.
	 * @param  boolean $skip_search Whether to skip searching and fetch all results.
	 * @return array   $results Search Results
	 */
	public static function qre_search_courses( $query = '', $skip_search = false ) {
		if ( ! is_user_logged_in() ) {
			return array();
		}
		$args = array(
			'post_type'      => 'sfwd-courses',
			'posts_per_page' => -1,
		);
		if ( false === $skip_search ) {
			$args['s'] = $query;
		}
		$courses = get_posts( $args );
		if ( empty( $courses ) ) {
			return $courses;
		}
		// Filter out items for which user doesn't have access.
		$courses = array_filter(
			$courses,
			function( $course ) {
				$current_user         = wp_get_current_user();
				$user_managed_courses = qre_get_user_managed_group_courses();

				$is_course_accessible = qre_check_if_course_accessible( $course->ID, $current_user, $user_managed_courses );
				if ( ! $is_course_accessible || is_wp_error( $is_course_accessible ) ) {
					return false;
				}
				return true;
			}
		);
		$results = array_map(
			function( $course ) use ( $query ) {
				$item = array(
					'post'  => $course,
					'title' => $course->post_title,
					'type'  => 'post',
					'ID'    => $course->ID,
					'icon'  => 'quiz_icon',
				);
				return $item;
			},
			$courses
		);
		return $results;
	}

	/**
	 * Function is used to get quizzes that match the queried string.
	 *
	 * @param  string  $query Search Query.
	 * @param  boolean $skip_search Whether to skip searching and fetch all results.
	 * @return array   $results Search Results
	 */
	public static function qre_search_quizzes( $query = '', $skip_search = false ) {
		if ( ! is_user_logged_in() ) {
			return array();
		}
		$args = array(
			'post_type'      => 'sfwd-quiz',
			'posts_per_page' => -1,
		);
		if ( false === $skip_search ) {
			$args['s'] = $query;
		}
		$quizzes = get_posts( $args );
		if ( empty( $quizzes ) ) {
			return $quizzes;
		}
		// Filter out items for which user doesn't have access.
		$quizzes = array_filter(
			$quizzes,
			function( $quiz ) {
				$current_user         = wp_get_current_user();
				$user_managed_courses = self::get_user_managed_group_courses();

				$is_quiz_accessible = self::check_if_quiz_accessible( $quiz->ID, $current_user, $user_managed_courses );
				if ( ! $is_quiz_accessible || is_wp_error( $is_quiz_accessible ) ) {
					return false;
				}
				return true;
			}
		);
		$results = array_map(
			function( $quiz ) use ( $query ) {
				$item = array(
					'title' => $quiz->post_title,
					'type'  => 'quiz',
					'ID'    => $quiz->ID,
					'icon'  => 'quiz_icon',
				);
				return $item;
			},
			$quizzes
		);
		return $results;
	}

	/**
	 * Function is used to get users that match the queried string.
	 *
	 * @param  string  $search_string Search Query.
	 * @param  boolean $skip_search Whether to skip searching and fetch all results.[WARNING: Skipping search isn't recommended. For systems that have large number of users, the system may crash due to memory shortage].
	 * @return array   $results Search Results.
	 */
	public static function qre_search_users( $search_string = '', $skip_search = false ) {
		if ( ! is_user_logged_in() ) {
			return array();
		}

		if ( false === $skip_search ) {
			$args        = array(
				'search'         => "*{$search_string}*",
				'search_columns' => array(
					'user_login',
					'user_nicename',
					'user_email',
					'display_name',
				),
				'meta_query'     => array(// WPCS: slow query ok.
					'relation' => 'OR',
					array(
						'key'     => 'first_name',
						'value'   => $search_string,
						'compare' => 'LIKE',
					),
					array(
						'key'     => 'last_name',
						'value'   => $search_string,
						'compare' => 'LIKE',
					),
				),
			);
			$users       = new WP_User_Query( $args );
			$users_found = (array) $users->get_results();
		} else {
			$users_found = get_users();
		}
		if ( empty( $users_found ) ) {
			return $users_found;
		}
		// Filter out items for which user doesn't have access.
		$users_found = array_filter(
			$users_found,
			function( $user ) {
				$current_user         = wp_get_current_user();
				$user_managed_courses = self::get_user_managed_group_courses();

				$is_user_accessible = self::check_if_user_accessible( $user->ID, $current_user, $user_managed_courses );
				if ( ! $is_user_accessible || is_wp_error( $is_user_accessible ) ) {
					return false;
				}
				return true;
			}
		);
		$results     = array_map(
			function( $user ) use ( $search_string ) {
				$item = array(
					'title' => $user->display_name,
					'type'  => 'user',
					'ID'    => $user->ID,
					'icon'  => '',
				);
				if ( empty( $search_string ) ) {
					return $item;
				}
				$item['icon'] = 'user_icon';
				return $item;
			},
			$users_found
		);
		return $results;
	}


	/**
	 * Get Courses of Groups where current user is the Group Leader.
	 *
	 * @return array User's Group affialiated Courses.
	 */
	public static function get_user_managed_group_courses() {
		global $group_leader_managed_courses;
		$courses = array();
		$user    = wp_get_current_user();
		if ( ! in_array( 'group_leader', (array) $user->roles ) ) {// phpcs:ignore
			return $courses;
		}

		if ( isset( $group_leader_managed_courses ) && ! empty( $group_leader_managed_courses ) ) {
			return $group_leader_managed_courses;
		}

		$associated_groups = learndash_get_administrators_group_ids( get_current_user_id() );

		if ( ! empty( $associated_groups ) ) {
			foreach ( $associated_groups as $associated_group ) {
				if ( empty( $courses ) ) {
					$courses = learndash_group_enrolled_courses( $associated_group );
				} else {
					$courses = array_merge( $courses, learndash_group_enrolled_courses( $associated_group ) );
				}
			}
		}
		$group_leader_managed_courses = $courses;
		return $courses;
	}


	/**
	 * This function is used to check whether the requested user info is accessible by the user.
	 *
	 * @param  integer $queried_obj_id       User ID.
	 * @param  WP_User $user                 User requesting resource.
	 * @param  array   $user_managed_courses User's Managed Courses(for Group Leaders).
	 * @return boolean/wp_error
	 */
	public static function check_if_user_accessible( $queried_obj_id, $user, $user_managed_courses ) {
		$user_id = self::check_if_user_ids_actually_present( $queried_obj_id );
		if ( empty( $user_id ) ) {
			return new \WP_Error( 400, __( 'Invalid Request. User ID doesn\'t exist.', 'quiz_reporting_learndash' ) );
		}
		// Check for Administrator.
		if ( current_user_can( 'manage_options' ) ) {
			return true;
		}
		// Check for self Data.
		if ( (int) $queried_obj_id === $user->ID ) {
			return true;
		}
		// Check for Group Leader.
		if ( ! empty( $user_managed_courses ) ) {
			$user_managed_groups = learndash_get_administrators_group_ids( $user->ID );
			if ( ! empty( $user_managed_groups ) ) {
				foreach ( $user_managed_groups as $group_id ) {
					if ( in_array( $queried_obj_id, learndash_get_groups_user_ids( $group_id ) ) ) { // phpcs:ignore
						return true;
					}
				}
			}
		}
		// Check for Instructor.
		if ( function_exists( 'ir_get_instructor_complete_course_list' ) && in_array( 'wdm_instructor', (array) $user->roles, true ) ) {
			$instructor_course_ids    = ir_get_instructor_complete_course_list( $user->ID );
			$user_enrolled_course_ids = learndash_user_get_enrolled_courses( $queried_obj_id, array(), true );
			if ( ! empty( array_intersect( $instructor_course_ids, $user_enrolled_course_ids ) ) ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Check for permanently deleted users.
	 *
	 * @param  string $user_ids User IDs to check.
	 * @return array  $user_ids_present Quiz IDs present.
	 */
	public static function check_if_user_ids_actually_present( $user_ids ) {
			global $wpdb;

			$table_name       = $wpdb->prefix . 'users';
			$user_ids_present = $wpdb->get_col( $wpdb->prepare( 'SELECT ID FROM ' . $wpdb->prefix . 'users WHERE ID IN (%s)', $user_ids ) );// WPCS: db call ok, cache ok.
			return $user_ids_present;
	}


	/**
	 * This function is used to check whether the requested quiz is accessible by the user.
	 *
	 * @param  integer $queried_obj_id       Quiz ID.
	 * @param  WP_User $user                 User requesting resource.
	 * @param  array   $user_managed_courses User's Managed Courses(for Group Leaders).
	 * @return boolean/wp_error
	 */
	public static function check_if_quiz_accessible( $queried_obj_id, $user, $user_managed_courses ) {
		$quiz_id = \WRLD_Quiz_Export_Db::instance()->check_if_quiz_ids_actually_present( $queried_obj_id );
		if ( empty( $quiz_id ) ) {
			return new \WP_Error( 400, __( 'Invalid Request. Quiz ID doesn\'t exist.', 'quiz_reporting_learndash' ) );
		}
		// Check for Administrator.
		if ( current_user_can( 'manage_options' ) ) {
			return true;
		}
		$queried_course_id = learndash_get_course_id( $queried_obj_id );
		// Check for Group Leader.
		if ( ! empty( $user_managed_courses ) ) {
			if ( in_array( $queried_course_id, $user_managed_courses ) ) {// phpcs:ignore
				return true;
			}
		}
		// Check for Instructor.
		if ( function_exists( 'ir_get_instructor_complete_course_list' ) && in_array( 'wdm_instructor', (array) $user->roles, true ) ) {
			$instructor_course_ids = ir_get_instructor_complete_course_list( $user->ID );
			if ( in_array( $queried_course_id, $instructor_course_ids ) ) {// phpcs:ignore
				return true;
			}
		}
		$user_enrolled_course_ids = learndash_user_get_enrolled_courses( $user->ID, array(), true );
		// Check for Subscriber.
		if ( in_array( $queried_course_id, $user_enrolled_course_ids ) ) {// phpcs:ignore
			return true;
		}
		// Check if stats exist regardless of course enrollment.
		$pro_quiz_id    = get_post_meta( $queried_obj_id, 'quiz_pro_id', true );
		$existing_stats = \WRLD_Quiz_Export_Db::instance()->get_all_statistic_ref_ids(
			array(
				'user_ids' => array( $user->ID ),
				'quiz_ids' => array( $pro_quiz_id ),
			)
		);
		if ( ! empty( $existing_stats ) ) {
			return true;
		}
		return false;
	}
}
